# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.blade_stability_analysis_calculation import BladeStabilityAnalysisCalculation
from dnv_bladed_models.blade_stability_analysis_idling import BladeStabilityAnalysisIdling
from dnv_bladed_models.blade_stability_analysis_mode_of_operation import BladeStabilityAnalysisModeOfOperation
from dnv_bladed_models.blade_stability_analysis_mode_of_operation_insert import BladeStabilityAnalysisModeOfOperationInsert
from dnv_bladed_models.blade_stability_analysis_parked import BladeStabilityAnalysisParked

from .schema_helper import SchemaHelper
from .models_impl import *

TBladeStabilityAnalysisModeOfOperationOptions = TypeVar('TBladeStabilityAnalysisModeOfOperationOptions', BladeStabilityAnalysisIdling, BladeStabilityAnalysisModeOfOperationInsert, BladeStabilityAnalysisParked, BladeStabilityAnalysisModeOfOperation, )

class BladeStabilityAnalysis(BladeStabilityAnalysisCalculation):
    r"""
    Defines a calculation which produces the damping and frequency of all the coupled rotor or blade modes plotted against wind speed for a single rotor.  This calculation allows analysis over a wider range of inflow conditions than the Cambell diagram calculation, rather than being constrained to normal operating conditions.
    
    Not supported yet.
    
    Attributes
    ----------
    SteadyCalculationType : Literal['BladeStabilityAnalysis'], default='BladeStabilityAnalysis', Not supported yet
        Defines the specific type of SteadyCalculation model in use.  For a `BladeStabilityAnalysis` object, this must always be set to a value of `BladeStabilityAnalysis`.
    
    ModeOfOperation : Union[BladeStabilityAnalysisIdling, BladeStabilityAnalysisModeOfOperationInsert, BladeStabilityAnalysisParked], Not supported yet
    
    Notes
    -----
    This model is not supported yet by the Bladed calculation engine.
    """
    SteadyCalculationType: Literal['BladeStabilityAnalysis'] = Field(alias="SteadyCalculationType", default='BladeStabilityAnalysis', allow_mutation=False, const=True) # Not supported yet # type: ignore
    ModeOfOperation: Union[BladeStabilityAnalysisIdling, BladeStabilityAnalysisModeOfOperationInsert, BladeStabilityAnalysisParked] = Field(alias="ModeOfOperation", default=None, discriminator='BladeStabilityAnalysisModeOfOperationType') # Not supported yet

    _relative_schema_path = 'SteadyCalculation/BladeStabilityAnalysis.json'
    _type_info = TypeInfo(
        set([('ModeOfOperation', 'BladeStabilityAnalysisModeOfOperationType'),]),
        set([]),
        set([]),
        'SteadyCalculationType').merge(BladeStabilityAnalysisCalculation._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    @property
    def ModeOfOperation_as_BladeStabilityAnalysisIdling(self) -> BladeStabilityAnalysisIdling:
        """
        Retrieves the value of ModeOfOperation guaranteeing it is a BladeStabilityAnalysisIdling; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        BladeStabilityAnalysisIdling
            A model object, guaranteed to be a BladeStabilityAnalysisIdling.

        Raises
        ------
        TypeError
            If the value is not a BladeStabilityAnalysisIdling.
        """
        return self.ModeOfOperation_as(BladeStabilityAnalysisIdling)


    @property
    def ModeOfOperation_as_BladeStabilityAnalysisParked(self) -> BladeStabilityAnalysisParked:
        """
        Retrieves the value of ModeOfOperation guaranteeing it is a BladeStabilityAnalysisParked; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        BladeStabilityAnalysisParked
            A model object, guaranteed to be a BladeStabilityAnalysisParked.

        Raises
        ------
        TypeError
            If the value is not a BladeStabilityAnalysisParked.
        """
        return self.ModeOfOperation_as(BladeStabilityAnalysisParked)


    @property
    def ModeOfOperation_as_inline(self) -> Union[BladeStabilityAnalysisIdling, BladeStabilityAnalysisParked]:
        """
        Retrieves the value of ModeOfOperation as a model object; if the value is specified with a '$insert', an error is raised.

        Returns
        -------
        Union[BladeStabilityAnalysisIdling, BladeStabilityAnalysisParked]
            A model object at the specified index, guaranteed to not be a '$insert'.

        Raises
        ------
        TypeError
            If the value is not one of the concrete types of BladeStabilityAnalysisModeOfOperation; i.e. it is specified with a '$insert'.
        """
        if isinstance(self.ModeOfOperation, BladeStabilityAnalysisModeOfOperationInsert) or self.ModeOfOperation.is_insert:
            raise TypeError(f"Expected ModeOfOperation value to be an in-line object, but it is currently in the '$insert' state.")
        return self.ModeOfOperation


    def ModeOfOperation_as(self, cls: Type[TBladeStabilityAnalysisModeOfOperationOptions])-> TBladeStabilityAnalysisModeOfOperationOptions:
        """
        Retrieves the value of ModeOfOperation, ensuring it is of the specified type.
        The specified type must be one of the valid concrete types of BladeStabilityAnalysisModeOfOperation, or the '$insert' type
        (in the case the model is defined to be inserted from an external resource).

        Useful when using type-checking development tools, as it provides a concise way to access a value with the correct type.

        Parameters
        ----------
        cls: Type[Union[BladeStabilityAnalysisIdling, BladeStabilityAnalysisModeOfOperationInsert, BladeStabilityAnalysisParked]]
            One of the valid concrete types of BladeStabilityAnalysisModeOfOperation, or the '$insert' type
            (in the case the model is defined to be inserted from an external resource).

        Returns
        -------
        TBladeStabilityAnalysisModeOfOperationOptions
            A model object of the specified type.

        Raises
        ------
        TypeError
            If the value is not of the specified type.

        Examples
        --------
        Get a reference to the value when it is one of the types of BladeStabilityAnalysisModeOfOperation:
        >>> val_obj = model_obj.ModeOfOperation_as(models.BladeStabilityAnalysisIdling)
        >>> val_obj = model_obj.ModeOfOperation_as(models.BladeStabilityAnalysisParked)

        Get a reference to the value, when it was specified with a '$insert' and read in from a file:
        >>> insert_obj = model_obj.ModeOfOperation_as(models.BladeStabilityAnalysisModeOfOperationInsert)
        """
        if not isinstance(self.ModeOfOperation, cls):
            raise TypeError(f"Expected ModeOfOperation of type '{cls.__name__}' but was type '{type(self.ModeOfOperation).__name__}'")
        return self.ModeOfOperation


    def _entity(self) -> bool:
        return True


BladeStabilityAnalysis.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.bladed_model import BladedModel
from dnv_bladed_models.current import Current
from dnv_bladed_models.current_insert import CurrentInsert
from dnv_bladed_models.currents import Currents
from dnv_bladed_models.jonswap_pierson_moskowitz import JonswapPiersonMoskowitz
from dnv_bladed_models.laminar_flow_current import LaminarFlowCurrent
from dnv_bladed_models.linear_airy import LinearAiry
from dnv_bladed_models.stream_function import StreamFunction
from dnv_bladed_models.tide import Tide
from dnv_bladed_models.turbulent_current import TurbulentCurrent
from dnv_bladed_models.user_defined_waves import UserDefinedWaves
from dnv_bladed_models.wave_spectrum import WaveSpectrum
from dnv_bladed_models.waves import Waves
from dnv_bladed_models.waves_insert import WavesInsert

from .schema_helper import SchemaHelper
from .models_impl import *

TCurrentOptions = TypeVar('TCurrentOptions', Currents, CurrentInsert, LaminarFlowCurrent, TurbulentCurrent, Current, )
TWavesOptions = TypeVar('TWavesOptions', WavesInsert, JonswapPiersonMoskowitz, LinearAiry, StreamFunction, UserDefinedWaves, WaveSpectrum, Waves, )

class SeaState(BladedModel):
    r"""
    The sea state at the turbine location.  The mean sea level is defined in the Turbine section of the data model.
    
    Not supported yet.
    
    Attributes
    ----------
    Current : Union[Currents, CurrentInsert, LaminarFlowCurrent, TurbulentCurrent], Not supported yet
    
    Waves : Union[WavesInsert, JonswapPiersonMoskowitz, LinearAiry, StreamFunction, UserDefinedWaves, WaveSpectrum], Not supported yet
    
    Tide : Tide, Not supported yet
    
    Notes
    -----
    This model is not supported yet by the Bladed calculation engine.
    """
    Current: Union[Currents, CurrentInsert, LaminarFlowCurrent, TurbulentCurrent] = Field(alias="Current", default=None, discriminator='CurrentType') # Not supported yet
    Waves: Union[WavesInsert, JonswapPiersonMoskowitz, LinearAiry, StreamFunction, UserDefinedWaves, WaveSpectrum] = Field(alias="Waves", default=None, discriminator='WavesType') # Not supported yet
    Tide: Tide = Field(alias="Tide", default=None) # Not supported yet

    _relative_schema_path = 'TimeDomainSimulation/Environment/SeaState/SeaState.json'
    _type_info = TypeInfo(
        set([('Current', 'CurrentType'),('Waves', 'WavesType'),]),
        set([]),
        set([]),
        None).merge(BladedModel._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    @property
    def Current_as_Currents(self) -> Currents:
        """
        Retrieves the value of Current guaranteeing it is a Currents; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        Currents
            A model object, guaranteed to be a Currents.

        Raises
        ------
        TypeError
            If the value is not a Currents.
        """
        return self.Current_as(Currents)


    @property
    def Current_as_LaminarFlowCurrent(self) -> LaminarFlowCurrent:
        """
        Retrieves the value of Current guaranteeing it is a LaminarFlowCurrent; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        LaminarFlowCurrent
            A model object, guaranteed to be a LaminarFlowCurrent.

        Raises
        ------
        TypeError
            If the value is not a LaminarFlowCurrent.
        """
        return self.Current_as(LaminarFlowCurrent)


    @property
    def Current_as_TurbulentCurrent(self) -> TurbulentCurrent:
        """
        Retrieves the value of Current guaranteeing it is a TurbulentCurrent; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        TurbulentCurrent
            A model object, guaranteed to be a TurbulentCurrent.

        Raises
        ------
        TypeError
            If the value is not a TurbulentCurrent.
        """
        return self.Current_as(TurbulentCurrent)


    @property
    def Current_as_inline(self) -> Union[Currents, LaminarFlowCurrent, TurbulentCurrent]:
        """
        Retrieves the value of Current as a model object; if the value is specified with a '$insert', an error is raised.

        Returns
        -------
        Union[Currents, LaminarFlowCurrent, TurbulentCurrent]
            A model object at the specified index, guaranteed to not be a '$insert'.

        Raises
        ------
        TypeError
            If the value is not one of the concrete types of Current; i.e. it is specified with a '$insert'.
        """
        if isinstance(self.Current, CurrentInsert) or self.Current.is_insert:
            raise TypeError(f"Expected Current value to be an in-line object, but it is currently in the '$insert' state.")
        return self.Current


    def Current_as(self, cls: Type[TCurrentOptions])-> TCurrentOptions:
        """
        Retrieves the value of Current, ensuring it is of the specified type.
        The specified type must be one of the valid concrete types of Current, or the '$insert' type
        (in the case the model is defined to be inserted from an external resource).

        Useful when using type-checking development tools, as it provides a concise way to access a value with the correct type.

        Parameters
        ----------
        cls: Type[Union[Currents, CurrentInsert, LaminarFlowCurrent, TurbulentCurrent]]
            One of the valid concrete types of Current, or the '$insert' type
            (in the case the model is defined to be inserted from an external resource).

        Returns
        -------
        TCurrentOptions
            A model object of the specified type.

        Raises
        ------
        TypeError
            If the value is not of the specified type.

        Examples
        --------
        Get a reference to the value when it is one of the types of Current:
        >>> val_obj = model_obj.Current_as(models.Currents)
        >>> val_obj = model_obj.Current_as(models.LaminarFlowCurrent)
        >>> val_obj = model_obj.Current_as(models.TurbulentCurrent)

        Get a reference to the value, when it was specified with a '$insert' and read in from a file:
        >>> insert_obj = model_obj.Current_as(models.CurrentInsert)
        """
        if not isinstance(self.Current, cls):
            raise TypeError(f"Expected Current of type '{cls.__name__}' but was type '{type(self.Current).__name__}'")
        return self.Current


    @property
    def Waves_as_JonswapPiersonMoskowitz(self) -> JonswapPiersonMoskowitz:
        """
        Retrieves the value of Waves guaranteeing it is a JonswapPiersonMoskowitz; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        JonswapPiersonMoskowitz
            A model object, guaranteed to be a JonswapPiersonMoskowitz.

        Raises
        ------
        TypeError
            If the value is not a JonswapPiersonMoskowitz.
        """
        return self.Waves_as(JonswapPiersonMoskowitz)


    @property
    def Waves_as_LinearAiry(self) -> LinearAiry:
        """
        Retrieves the value of Waves guaranteeing it is a LinearAiry; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        LinearAiry
            A model object, guaranteed to be a LinearAiry.

        Raises
        ------
        TypeError
            If the value is not a LinearAiry.
        """
        return self.Waves_as(LinearAiry)


    @property
    def Waves_as_StreamFunction(self) -> StreamFunction:
        """
        Retrieves the value of Waves guaranteeing it is a StreamFunction; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        StreamFunction
            A model object, guaranteed to be a StreamFunction.

        Raises
        ------
        TypeError
            If the value is not a StreamFunction.
        """
        return self.Waves_as(StreamFunction)


    @property
    def Waves_as_UserDefinedWaves(self) -> UserDefinedWaves:
        """
        Retrieves the value of Waves guaranteeing it is a UserDefinedWaves; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        UserDefinedWaves
            A model object, guaranteed to be a UserDefinedWaves.

        Raises
        ------
        TypeError
            If the value is not a UserDefinedWaves.
        """
        return self.Waves_as(UserDefinedWaves)


    @property
    def Waves_as_WaveSpectrum(self) -> WaveSpectrum:
        """
        Retrieves the value of Waves guaranteeing it is a WaveSpectrum; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        WaveSpectrum
            A model object, guaranteed to be a WaveSpectrum.

        Raises
        ------
        TypeError
            If the value is not a WaveSpectrum.
        """
        return self.Waves_as(WaveSpectrum)


    @property
    def Waves_as_inline(self) -> Union[JonswapPiersonMoskowitz, LinearAiry, StreamFunction, UserDefinedWaves, WaveSpectrum]:
        """
        Retrieves the value of Waves as a model object; if the value is specified with a '$insert', an error is raised.

        Returns
        -------
        Union[JonswapPiersonMoskowitz, LinearAiry, StreamFunction, UserDefinedWaves, WaveSpectrum]
            A model object at the specified index, guaranteed to not be a '$insert'.

        Raises
        ------
        TypeError
            If the value is not one of the concrete types of Waves; i.e. it is specified with a '$insert'.
        """
        if isinstance(self.Waves, WavesInsert) or self.Waves.is_insert:
            raise TypeError(f"Expected Waves value to be an in-line object, but it is currently in the '$insert' state.")
        return self.Waves


    def Waves_as(self, cls: Type[TWavesOptions])-> TWavesOptions:
        """
        Retrieves the value of Waves, ensuring it is of the specified type.
        The specified type must be one of the valid concrete types of Waves, or the '$insert' type
        (in the case the model is defined to be inserted from an external resource).

        Useful when using type-checking development tools, as it provides a concise way to access a value with the correct type.

        Parameters
        ----------
        cls: Type[Union[WavesInsert, JonswapPiersonMoskowitz, LinearAiry, StreamFunction, UserDefinedWaves, WaveSpectrum]]
            One of the valid concrete types of Waves, or the '$insert' type
            (in the case the model is defined to be inserted from an external resource).

        Returns
        -------
        TWavesOptions
            A model object of the specified type.

        Raises
        ------
        TypeError
            If the value is not of the specified type.

        Examples
        --------
        Get a reference to the value when it is one of the types of Waves:
        >>> val_obj = model_obj.Waves_as(models.JonswapPiersonMoskowitz)
        >>> val_obj = model_obj.Waves_as(models.LinearAiry)
        >>> val_obj = model_obj.Waves_as(models.StreamFunction)
        >>> val_obj = model_obj.Waves_as(models.UserDefinedWaves)
        >>> val_obj = model_obj.Waves_as(models.WaveSpectrum)

        Get a reference to the value, when it was specified with a '$insert' and read in from a file:
        >>> insert_obj = model_obj.Waves_as(models.WavesInsert)
        """
        if not isinstance(self.Waves, cls):
            raise TypeError(f"Expected Waves of type '{cls.__name__}' but was type '{type(self.Waves).__name__}'")
        return self.Waves


    def _entity(self) -> bool:
        return True


SeaState.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from abc import ABC

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.bladed_model import BladedModel

from .schema_helper import SchemaHelper
from .models_impl import *


class PerturbationSettings(BladedModel, ABC):
    r"""
    
    
    Not supported yet.
    
    Attributes
    ----------
    NumberOfPerturbationPoints : int, default=5, Not supported yet
        The number of values that each state gets perturbed to either side of the equilibrium point.  Larger numbers provide more robustness; smaller numbers provide faster analysis and lower memory usage.
    
    AbsoluteTolerancePerturbationScale : float, default=100, Not supported yet
        The zero value states have a perturbation magnitude of this value multiplied by the absolute tolerance of the state
    
    StateRelativePerturbation : float, default=0.005, Not supported yet
        The magnitude of the state perturbations relative to the absolute steady-state state values
    
    Notes
    -----
    This model is not supported yet by the Bladed calculation engine.
    """
    NumberOfPerturbationPoints: int = Field(alias="NumberOfPerturbationPoints", default=None) # Not supported yet
    AbsoluteTolerancePerturbationScale: float = Field(alias="AbsoluteTolerancePerturbationScale", default=None) # Not supported yet
    StateRelativePerturbation: float = Field(alias="StateRelativePerturbation", default=None) # Not supported yet

    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        None).merge(BladedModel._type_info)


PerturbationSettings.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.fault import Fault

from .schema_helper import SchemaHelper
from .models_impl import *


class ControllerFault(Fault):
    r"""
    A controller fault occuring at a specified time.  A signal will be sent to the controller, and the controller will be expected to implement the fault behaviour.
    
    Attributes
    ----------
    EventType : Literal['ControllerFault'], default='ControllerFault'
        Defines the specific type of Event model in use.  For a `ControllerFault` object, this must always be set to a value of `ControllerFault`.
    
    FaultCode : int
        A fault number that will be provided to the controller.  It is for the controller to decide how to react to this number.
    
    FaultParametersAsString : str
        A string and passed to the external controller at the time the fault occurs.
    
    FaultParametersAsJson : Dict[str, Any]
        A JSON object that will be serialised as a string and passed to the external controller at the time the fault occurs.
    
    Notes
    -----
    
    """
    EventType: Literal['ControllerFault'] = Field(alias="EventType", default='ControllerFault', allow_mutation=False, const=True) # type: ignore
    FaultCode: int = Field(alias="FaultCode", default=None)
    FaultParametersAsString: str = Field(alias="FaultParametersAsString", default=None)
    FaultParametersAsJson: Dict[str, Any] = Field(alias="FaultParametersAsJson", default=None)

    _relative_schema_path = 'TimeDomainSimulation/Event/ControllerFault.json'
    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        'EventType').merge(Fault._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    def _entity(self) -> bool:
        return True


ControllerFault.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.bladed_model import BladedModel
from dnv_bladed_models.signal_properties_acceleration import SignalPropertiesAcceleration
from dnv_bladed_models.signal_properties_angle import SignalPropertiesAngle
from dnv_bladed_models.signal_properties_angular_acceleration import SignalPropertiesAngularAcceleration
from dnv_bladed_models.signal_properties_angular_velocity import SignalPropertiesAngularVelocity
from dnv_bladed_models.signal_properties_force import SignalPropertiesForce
from dnv_bladed_models.signal_properties_length import SignalPropertiesLength
from dnv_bladed_models.signal_properties_moment import SignalPropertiesMoment
from dnv_bladed_models.signal_properties_power import SignalPropertiesPower
from dnv_bladed_models.signal_properties_velocity import SignalPropertiesVelocity

from .schema_helper import SchemaHelper
from .models_impl import *


class MeasuredSignalProperties(BladedModel):
    r"""
    The noise and transducer properties for those signals representing values coming from a physical sensor.
    
    Not supported yet.
    
    Attributes
    ----------
    RandomNumberSeed : int, default=0, Not supported yet
        A seed for the random number generator to ensure that subsequent runs have identical noise signatures.
    
    TurnOffNoise : bool, default=False, Not supported yet
        This allows the noise to be turned off globally.  Note: this turns off noise, but keeps discretisation, sampling time, faults and transducer behaviour.
    
    ShaftPowerSignals : SignalPropertiesPower, Not supported yet
    
    RotorSpeedSignals : SignalPropertiesAngularVelocity, Not supported yet
    
    ElectricalPowerOutputSignals : SignalPropertiesPower, Not supported yet
    
    GeneratorSpeedSignals : SignalPropertiesAngularVelocity, Not supported yet
    
    GeneratorTorqueSignals : SignalPropertiesMoment, Not supported yet
    
    YawBearingAngularPositionSignals : SignalPropertiesAngle, Not supported yet
    
    YawBearingAngularVelocitySignals : SignalPropertiesAngularVelocity, Not supported yet
    
    YawBearingAngularAccelerationSignals : SignalPropertiesAngularAcceleration, Not supported yet
    
    YawMotorRateSignals : SignalPropertiesAngularVelocity, Not supported yet
    
    YawErrorSignals : SignalPropertiesAngle, Not supported yet
    
    NacelleAngleFromNorthSignals : SignalPropertiesAngle, Not supported yet
    
    TowerTopForeAftAccelerationSignals : SignalPropertiesAcceleration, Not supported yet
    
    TowerTopSideSideAccelerationSignals : SignalPropertiesAcceleration, Not supported yet
    
    ShaftTorqueSignals : SignalPropertiesMoment, Not supported yet
    
    YawBearingMySignals : SignalPropertiesMoment, Not supported yet
    
    YawBearingMzSignals : SignalPropertiesMoment, Not supported yet
    
    NacelleRollAngleSignals : SignalPropertiesAngle, Not supported yet
    
    NacelleNoddingAngleSignals : SignalPropertiesAngle, Not supported yet
    
    NacelleRollAccelerationSignals : SignalPropertiesAngularAcceleration, Not supported yet
    
    NacelleNoddingAccelerationSignals : SignalPropertiesAngularAcceleration, Not supported yet
    
    NacelleYawAccelerationSignals : SignalPropertiesAngularAcceleration, Not supported yet
    
    RotorAzimuthAngleSignals : SignalPropertiesAngle, Not supported yet
    
    NominalHubFlowSpeedSignals : SignalPropertiesVelocity, Not supported yet
    
    RotatingHubMySignals : SignalPropertiesMoment, Not supported yet
    
    RotatingHubMzSignals : SignalPropertiesMoment, Not supported yet
    
    FixedHubMySignals : SignalPropertiesMoment, Not supported yet
    
    FixedHubMzSignals : SignalPropertiesMoment, Not supported yet
    
    FixedHubFxSignals : SignalPropertiesForce, Not supported yet
    
    FixedHubFySignals : SignalPropertiesForce, Not supported yet
    
    FixedHubFzSignals : SignalPropertiesForce, Not supported yet
    
    PitchAngleSignals : SignalPropertiesAngle, Not supported yet
    
    PitchRateSignals : SignalPropertiesAngularVelocity, Not supported yet
    
    PitchActuatorTorqueSignals : SignalPropertiesMoment, Not supported yet
    
    PitchBearingFrictionSignals : SignalPropertiesMoment, Not supported yet
    
    PitchBearingStictionSignals : SignalPropertiesMoment, Not supported yet
    
    BladeOutOfPlaneBendingMomentSignals : SignalPropertiesMoment, Not supported yet
    
    BladeInPlaneBendingMomentSignals : SignalPropertiesMoment, Not supported yet
    
    PitchBearingMxSignals : SignalPropertiesMoment, Not supported yet
    
    PitchBearingMySignals : SignalPropertiesMoment, Not supported yet
    
    PitchBearingMzSignals : SignalPropertiesMoment, Not supported yet
    
    PitchBearingRadialForceSignals : SignalPropertiesForce, Not supported yet
    
    PitchBearingAxialForceSignals : SignalPropertiesForce, Not supported yet
    
    PitchBearingFxSignals : SignalPropertiesForce, Not supported yet
    
    PitchBearingFySignals : SignalPropertiesForce, Not supported yet
    
    BladeStationWindSpeedSignals : SignalPropertiesVelocity, Not supported yet
    
    BladeStationAngleOfAttackSignals : SignalPropertiesAngle, Not supported yet
    
    AileronAngleSignals : SignalPropertiesAngle, Not supported yet
    
    AileronRateSignals : SignalPropertiesAngularVelocity, Not supported yet
    
    BladeStationPositionXSignals : SignalPropertiesLength, Not supported yet
    
    BladeStationPositionYSignals : SignalPropertiesLength, Not supported yet
    
    BladeStationPositionZSignals : SignalPropertiesLength, Not supported yet
    
    BladeStationPositionXRotationSignals : SignalPropertiesAngle, Not supported yet
    
    BladeStationPositionYRotationSignals : SignalPropertiesAngle, Not supported yet
    
    BladeStationPositionZRotationSignals : SignalPropertiesAngle, Not supported yet
    
    LidarBeamFocalPointVelocitySignals : SignalPropertiesVelocity, Not supported yet
    
    Notes
    -----
    This model is not supported yet by the Bladed calculation engine.
    """
    RandomNumberSeed: int = Field(alias="RandomNumberSeed", default=None) # Not supported yet
    TurnOffNoise: bool = Field(alias="TurnOffNoise", default=None) # Not supported yet
    ShaftPowerSignals: SignalPropertiesPower = Field(alias="ShaftPowerSignals", default=None) # Not supported yet
    RotorSpeedSignals: SignalPropertiesAngularVelocity = Field(alias="RotorSpeedSignals", default=None) # Not supported yet
    ElectricalPowerOutputSignals: SignalPropertiesPower = Field(alias="ElectricalPowerOutputSignals", default=None) # Not supported yet
    GeneratorSpeedSignals: SignalPropertiesAngularVelocity = Field(alias="GeneratorSpeedSignals", default=None) # Not supported yet
    GeneratorTorqueSignals: SignalPropertiesMoment = Field(alias="GeneratorTorqueSignals", default=None) # Not supported yet
    YawBearingAngularPositionSignals: SignalPropertiesAngle = Field(alias="YawBearingAngularPositionSignals", default=None) # Not supported yet
    YawBearingAngularVelocitySignals: SignalPropertiesAngularVelocity = Field(alias="YawBearingAngularVelocitySignals", default=None) # Not supported yet
    YawBearingAngularAccelerationSignals: SignalPropertiesAngularAcceleration = Field(alias="YawBearingAngularAccelerationSignals", default=None) # Not supported yet
    YawMotorRateSignals: SignalPropertiesAngularVelocity = Field(alias="YawMotorRateSignals", default=None) # Not supported yet
    YawErrorSignals: SignalPropertiesAngle = Field(alias="YawErrorSignals", default=None) # Not supported yet
    NacelleAngleFromNorthSignals: SignalPropertiesAngle = Field(alias="NacelleAngleFromNorthSignals", default=None) # Not supported yet
    TowerTopForeAftAccelerationSignals: SignalPropertiesAcceleration = Field(alias="TowerTopForeAftAccelerationSignals", default=None) # Not supported yet
    TowerTopSideSideAccelerationSignals: SignalPropertiesAcceleration = Field(alias="TowerTopSideSideAccelerationSignals", default=None) # Not supported yet
    ShaftTorqueSignals: SignalPropertiesMoment = Field(alias="ShaftTorqueSignals", default=None) # Not supported yet
    YawBearingMySignals: SignalPropertiesMoment = Field(alias="YawBearingMySignals", default=None) # Not supported yet
    YawBearingMzSignals: SignalPropertiesMoment = Field(alias="YawBearingMzSignals", default=None) # Not supported yet
    NacelleRollAngleSignals: SignalPropertiesAngle = Field(alias="NacelleRollAngleSignals", default=None) # Not supported yet
    NacelleNoddingAngleSignals: SignalPropertiesAngle = Field(alias="NacelleNoddingAngleSignals", default=None) # Not supported yet
    NacelleRollAccelerationSignals: SignalPropertiesAngularAcceleration = Field(alias="NacelleRollAccelerationSignals", default=None) # Not supported yet
    NacelleNoddingAccelerationSignals: SignalPropertiesAngularAcceleration = Field(alias="NacelleNoddingAccelerationSignals", default=None) # Not supported yet
    NacelleYawAccelerationSignals: SignalPropertiesAngularAcceleration = Field(alias="NacelleYawAccelerationSignals", default=None) # Not supported yet
    RotorAzimuthAngleSignals: SignalPropertiesAngle = Field(alias="RotorAzimuthAngleSignals", default=None) # Not supported yet
    NominalHubFlowSpeedSignals: SignalPropertiesVelocity = Field(alias="NominalHubFlowSpeedSignals", default=None) # Not supported yet
    RotatingHubMySignals: SignalPropertiesMoment = Field(alias="RotatingHubMySignals", default=None) # Not supported yet
    RotatingHubMzSignals: SignalPropertiesMoment = Field(alias="RotatingHubMzSignals", default=None) # Not supported yet
    FixedHubMySignals: SignalPropertiesMoment = Field(alias="FixedHubMySignals", default=None) # Not supported yet
    FixedHubMzSignals: SignalPropertiesMoment = Field(alias="FixedHubMzSignals", default=None) # Not supported yet
    FixedHubFxSignals: SignalPropertiesForce = Field(alias="FixedHubFxSignals", default=None) # Not supported yet
    FixedHubFySignals: SignalPropertiesForce = Field(alias="FixedHubFySignals", default=None) # Not supported yet
    FixedHubFzSignals: SignalPropertiesForce = Field(alias="FixedHubFzSignals", default=None) # Not supported yet
    PitchAngleSignals: SignalPropertiesAngle = Field(alias="PitchAngleSignals", default=None) # Not supported yet
    PitchRateSignals: SignalPropertiesAngularVelocity = Field(alias="PitchRateSignals", default=None) # Not supported yet
    PitchActuatorTorqueSignals: SignalPropertiesMoment = Field(alias="PitchActuatorTorqueSignals", default=None) # Not supported yet
    PitchBearingFrictionSignals: SignalPropertiesMoment = Field(alias="PitchBearingFrictionSignals", default=None) # Not supported yet
    PitchBearingStictionSignals: SignalPropertiesMoment = Field(alias="PitchBearingStictionSignals", default=None) # Not supported yet
    BladeOutOfPlaneBendingMomentSignals: SignalPropertiesMoment = Field(alias="BladeOutOfPlaneBendingMomentSignals", default=None) # Not supported yet
    BladeInPlaneBendingMomentSignals: SignalPropertiesMoment = Field(alias="BladeInPlaneBendingMomentSignals", default=None) # Not supported yet
    PitchBearingMxSignals: SignalPropertiesMoment = Field(alias="PitchBearingMxSignals", default=None) # Not supported yet
    PitchBearingMySignals: SignalPropertiesMoment = Field(alias="PitchBearingMySignals", default=None) # Not supported yet
    PitchBearingMzSignals: SignalPropertiesMoment = Field(alias="PitchBearingMzSignals", default=None) # Not supported yet
    PitchBearingRadialForceSignals: SignalPropertiesForce = Field(alias="PitchBearingRadialForceSignals", default=None) # Not supported yet
    PitchBearingAxialForceSignals: SignalPropertiesForce = Field(alias="PitchBearingAxialForceSignals", default=None) # Not supported yet
    PitchBearingFxSignals: SignalPropertiesForce = Field(alias="PitchBearingFxSignals", default=None) # Not supported yet
    PitchBearingFySignals: SignalPropertiesForce = Field(alias="PitchBearingFySignals", default=None) # Not supported yet
    BladeStationWindSpeedSignals: SignalPropertiesVelocity = Field(alias="BladeStationWindSpeedSignals", default=None) # Not supported yet
    BladeStationAngleOfAttackSignals: SignalPropertiesAngle = Field(alias="BladeStationAngleOfAttackSignals", default=None) # Not supported yet
    AileronAngleSignals: SignalPropertiesAngle = Field(alias="AileronAngleSignals", default=None) # Not supported yet
    AileronRateSignals: SignalPropertiesAngularVelocity = Field(alias="AileronRateSignals", default=None) # Not supported yet
    BladeStationPositionXSignals: SignalPropertiesLength = Field(alias="BladeStationPositionXSignals", default=None) # Not supported yet
    BladeStationPositionYSignals: SignalPropertiesLength = Field(alias="BladeStationPositionYSignals", default=None) # Not supported yet
    BladeStationPositionZSignals: SignalPropertiesLength = Field(alias="BladeStationPositionZSignals", default=None) # Not supported yet
    BladeStationPositionXRotationSignals: SignalPropertiesAngle = Field(alias="BladeStationPositionXRotationSignals", default=None) # Not supported yet
    BladeStationPositionYRotationSignals: SignalPropertiesAngle = Field(alias="BladeStationPositionYRotationSignals", default=None) # Not supported yet
    BladeStationPositionZRotationSignals: SignalPropertiesAngle = Field(alias="BladeStationPositionZRotationSignals", default=None) # Not supported yet
    LidarBeamFocalPointVelocitySignals: SignalPropertiesVelocity = Field(alias="LidarBeamFocalPointVelocitySignals", default=None) # Not supported yet

    _relative_schema_path = 'Turbine/BladedControl/MeasuredSignalProperties/MeasuredSignalProperties.json'
    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        None).merge(BladedModel._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    def _entity(self) -> bool:
        return True


MeasuredSignalProperties.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.bladed_model import BladedModel
from dnv_bladed_models.external_controller import ExternalController
from dnv_bladed_models.measured_signal_properties import MeasuredSignalProperties
from dnv_bladed_models.safety_system import SafetySystem

from .schema_helper import SchemaHelper
from .models_impl import *


class BladedControl(BladedModel):
    r"""
    A definition of how the turbine is controlled.
    
    Attributes
    ----------
    SignalProperties : MeasuredSignalProperties, Not supported yet
    
    ControlDiscreteTimeStep : float, default=0.1
        The minimum timestep to be used across all of the controllers.  Controllers do not need to be called on every timestep (see the 'TimeStepMultiplier' parameter, below).
    
    Controllers : List[ExternalController]
        A list of externally-defined controllers.  These will be run in the order they are specified, but can be run at different multiples of the timestep set above.  If no controllers are specified, there will be no control action taken throughout the simulation.  No control demands will be calculated or acted upon, although the rotor speed may well accelerate.  The friction of the pitch systems may also be overcome, resulting in a change in the pitch angles.
    
    SafetySystem : SafetySystem, Not supported yet
    
    Notes
    -----
    
    """
    SignalProperties: MeasuredSignalProperties = Field(alias="SignalProperties", default=None) # Not supported yet
    ControlDiscreteTimeStep: float = Field(alias="ControlDiscreteTimeStep", default=None)
    Controllers: List[ExternalController] = Field(alias="Controllers", default=list())
    SafetySystem: SafetySystem = Field(alias="SafetySystem", default=None) # Not supported yet

    _relative_schema_path = 'Turbine/BladedControl/BladedControl.json'
    _type_info = TypeInfo(
        set([]),
        set([]),
        set(['Controllers',]),
        None).merge(BladedModel._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    def _entity(self) -> bool:
        return True


BladedControl.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.steady_calculation import SteadyCalculation
from dnv_bladed_models.steady_calculation_with_component_outputs import SteadyCalculationWithComponentOutputs
from dnv_bladed_models.velocity_range import VelocityRange

from .schema_helper import SchemaHelper
from .models_impl import *


class SteadyOperationalLoadsCalculation(SteadyCalculation):
    r"""
    Defines a calculation which produces all the loads in a steady, uniform wind field.  The entire turbine is modelled for this analysis and prebend and sweep included, as well as flexibility.  Most realities such as tower shadow and wind shear are ignored.
    
    Attributes
    ----------
    SteadyCalculationType : Literal['SteadyOperationalLoads'], default='SteadyOperationalLoads'
        Defines the specific type of SteadyCalculation model in use.  For a `SteadyOperationalLoads` object, this must always be set to a value of `SteadyOperationalLoads`.
    
    WindSpeedRange : VelocityRange
    
    Outputs : SteadyCalculationWithComponentOutputs
    
    Notes
    -----
    
    """
    SteadyCalculationType: Literal['SteadyOperationalLoads'] = Field(alias="SteadyCalculationType", default='SteadyOperationalLoads', allow_mutation=False, const=True) # type: ignore
    WindSpeedRange: VelocityRange = Field(alias="WindSpeedRange", default=None)
    Outputs: SteadyCalculationWithComponentOutputs = Field(alias="Outputs", default=None)

    _relative_schema_path = 'SteadyCalculation/SteadyOperationalLoadsCalculation.json'
    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        'SteadyCalculationType').merge(SteadyCalculation._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    def _entity(self) -> bool:
        return True


SteadyOperationalLoadsCalculation.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.component import Component
from dnv_bladed_models.rotational_stiffness import RotationalStiffness
from dnv_bladed_models.translational_stiffness import TranslationalStiffness

from .schema_helper import SchemaHelper
from .models_impl import *


class Flexibility(Component):
    r"""
    A generic component representing a flexibility between two other nodes.  The proximal and distal nodes can have flexibility in up to six degrees of freedom.  The hinge is assumed to be rigid in any direction where the properties are not specified.
    
    Attributes
    ----------
    ComponentType : Literal['Flexibility'], default='Flexibility'
        Defines the specific type of Component model in use.  For a `Flexibility` object, this must always be set to a value of `Flexibility`.
    
    AlongX : TranslationalStiffness
    
    AlongY : TranslationalStiffness
    
    AlongZ : TranslationalStiffness
    
    AboutX : RotationalStiffness
    
    AboutY : RotationalStiffness
    
    AboutZ : RotationalStiffness
    
    Notes
    -----
    
    """
    ComponentType: Literal['Flexibility'] = Field(alias="ComponentType", default='Flexibility', allow_mutation=False, const=True) # type: ignore
    AlongX: TranslationalStiffness = Field(alias="AlongX", default=None)
    AlongY: TranslationalStiffness = Field(alias="AlongY", default=None)
    AlongZ: TranslationalStiffness = Field(alias="AlongZ", default=None)
    AboutX: RotationalStiffness = Field(alias="AboutX", default=None)
    AboutY: RotationalStiffness = Field(alias="AboutY", default=None)
    AboutZ: RotationalStiffness = Field(alias="AboutZ", default=None)

    _relative_schema_path = 'Components/Flexibility.json'
    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        'ComponentType').merge(Component._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    def _entity(self) -> bool:
        return True


Flexibility.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from abc import ABC

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.bladed_model import BladedModel
class Outputs_FileFormatEnum(str, Enum):
    BINARY = "BINARY"
    ASCII = "ASCII"

from .schema_helper import SchemaHelper
from .models_impl import *


class Outputs(BladedModel, ABC):
    r"""
    Definition outputs to write for this analysis.
    
    Attributes
    ----------
    OutputDirectory : str
        The output directory for results files.
    
    FileStem : str, default=''
        A name for the analysis that will be used as the name of all the output files.  If omitted, the run type will be used, such as 'powprod'.
    
    FileFormat : Outputs_FileFormatEnum, default='BINARY'
        The output format, whether it is ASCII or binary.
    
    OutputDongleActivity : bool, default=False
        If true, the dongle activity will be logged.
    
    Notes
    -----
    
    """
    OutputDirectory: str = Field(alias="OutputDirectory", default=None)
    FileStem: str = Field(alias="FileStem", default=None)
    FileFormat: Outputs_FileFormatEnum = Field(alias="FileFormat", default=None)
    OutputDongleActivity: bool = Field(alias="OutputDongleActivity", default=None)

    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        None).merge(BladedModel._type_info)


Outputs.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from abc import ABC

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.bladed_model import BladedModel
class DynamicWake_DynamicWakeTypeEnum(str, Enum):
    EQUILIBRIUM_WAKE_MODEL = "EquilibriumWakeModel"
    FREE_FLOW_MODEL = "FreeFlowModel"
    FROZEN_WAKE_MODEL = "FrozenWakeModel"
    OYE_DYNAMIC_WAKE = "OyeDynamicWake"
    PITT_AND_PETERS_MODEL = "PittAndPetersModel"
    INSERT = "Insert"
class DynamicWake_AreaAveragingMethodEnum(str, Enum):
    OVER_ANNULUS = "OVER_ANNULUS"
    NONE = "NONE"

from .schema_helper import SchemaHelper
from .models_impl import *


class DynamicWake(BladedModel, ABC):
    r"""
    Common properties for all dynamic wake models.
    
    Attributes
    ----------
    DynamicWakeType : DynamicWake_DynamicWakeTypeEnum
        Defines the specific type of model in use.
    
    AreaAveragingMethod : DynamicWake_AreaAveragingMethodEnum, default='OVER_ANNULUS'
        With the \"over annulus\" method, the dynamic wake is calculated over the entire annular ring.  Induced velocities are averaged over the number of blades.  If \"none\" is selected, the annulus is divided into segments to which separate dynamic wakes are applied.
    
    Notes
    -----
    
    This class is an abstraction, with the following concrete implementations:
        - EquilibriumWakeModel
        - FreeFlowModel
        - FrozenWakeModel
        - DynamicWakeInsert
        - OyeDynamicWake
        - PittAndPetersModel
    
    """
    DynamicWakeType: DynamicWake_DynamicWakeTypeEnum = Field(alias="DynamicWakeType", default=None)
    AreaAveragingMethod: DynamicWake_AreaAveragingMethodEnum = Field(alias="AreaAveragingMethod", default=None)

    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        None).merge(BladedModel._type_info)


DynamicWake.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.regular_waves import RegularWaves

from .schema_helper import SchemaHelper
from .models_impl import *


class StreamFunction(RegularWaves):
    r"""
    The definition of Stream Function waves.
    
    Not supported yet.
    
    Attributes
    ----------
    WavesType : Literal['StreamFunction'], default='StreamFunction', Not supported yet
        Defines the specific type of Waves model in use.  For a `StreamFunction` object, this must always be set to a value of `StreamFunction`.
    
    UseTotalAcceleration : bool, Not supported yet
        If true, total acceleration (convective and time derivative) will be used for Morison loading when using the stream function.  The water particle accelerations in Bladed only include the time derivative terms. The convective terms of the water particles can be added when using the stream function for the water particle acceleration calculations.  If irregular waves and additional constrained waves are applied, where the constrained wave is defined by a stream function and total acceleration is turned on, the water particle acceleration will be calculated using local derivatives for the irregular waves and substantive derivatives for the constrained waves and will therefore be inconsistent.
    
    Notes
    -----
    This model is not supported yet by the Bladed calculation engine.
    """
    WavesType: Literal['StreamFunction'] = Field(alias="WavesType", default='StreamFunction', allow_mutation=False, const=True) # Not supported yet # type: ignore
    UseTotalAcceleration: bool = Field(alias="UseTotalAcceleration", default=None) # Not supported yet

    _relative_schema_path = 'TimeDomainSimulation/Environment/SeaState/Waves/StreamFunction.json'
    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        'WavesType').merge(RegularWaves._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    def _entity(self) -> bool:
        return True


StreamFunction.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from abc import ABC

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.component import Component
from dnv_bladed_models.electrical_losses import ElectricalLosses
from dnv_bladed_models.electrical_losses_insert import ElectricalLossesInsert
from dnv_bladed_models.generator_output_group_library import GeneratorOutputGroupLibrary
from dnv_bladed_models.linear_electrical_losses import LinearElectricalLosses
from dnv_bladed_models.non_linear_electrical_losses import NonLinearElectricalLosses

from .schema_helper import SchemaHelper
from .models_impl import *

TElectricalLossesOptions = TypeVar('TElectricalLossesOptions', ElectricalLossesInsert, LinearElectricalLosses, NonLinearElectricalLosses, ElectricalLosses, )

class Generator(Component, ABC):
    r"""
    The common properties for all types of generator.
    
    Attributes
    ----------
    GeneratorInertia : float
        The total rotational inertia of the generator, including that of the high-speed shaft after the clutch.
    
    Losses : Union[ElectricalLossesInsert, LinearElectricalLosses, NonLinearElectricalLosses]
    
    OutputGroups : GeneratorOutputGroupLibrary, Not supported yet
    
    Notes
    -----
    
    """
    GeneratorInertia: float = Field(alias="GeneratorInertia", default=None)
    Losses: Union[ElectricalLossesInsert, LinearElectricalLosses, NonLinearElectricalLosses] = Field(alias="Losses", default=None, discriminator='ElectricalLossesType')
    OutputGroups: GeneratorOutputGroupLibrary = Field(alias="OutputGroups", default=GeneratorOutputGroupLibrary()) # Not supported yet

    _type_info = TypeInfo(
        set([('Losses', 'ElectricalLossesType'),]),
        set([]),
        set(['OutputGroups',]),
        None).merge(Component._type_info)


    @property
    def Losses_as_LinearElectricalLosses(self) -> LinearElectricalLosses:
        """
        Retrieves the value of Losses guaranteeing it is a LinearElectricalLosses; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        LinearElectricalLosses
            A model object, guaranteed to be a LinearElectricalLosses.

        Raises
        ------
        TypeError
            If the value is not a LinearElectricalLosses.
        """
        return self.Losses_as(LinearElectricalLosses)


    @property
    def Losses_as_NonLinearElectricalLosses(self) -> NonLinearElectricalLosses:
        """
        Retrieves the value of Losses guaranteeing it is a NonLinearElectricalLosses; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        NonLinearElectricalLosses
            A model object, guaranteed to be a NonLinearElectricalLosses.

        Raises
        ------
        TypeError
            If the value is not a NonLinearElectricalLosses.
        """
        return self.Losses_as(NonLinearElectricalLosses)


    @property
    def Losses_as_inline(self) -> Union[LinearElectricalLosses, NonLinearElectricalLosses]:
        """
        Retrieves the value of Losses as a model object; if the value is specified with a '$insert', an error is raised.

        Returns
        -------
        Union[LinearElectricalLosses, NonLinearElectricalLosses]
            A model object at the specified index, guaranteed to not be a '$insert'.

        Raises
        ------
        TypeError
            If the value is not one of the concrete types of ElectricalLosses; i.e. it is specified with a '$insert'.
        """
        if isinstance(self.Losses, ElectricalLossesInsert) or self.Losses.is_insert:
            raise TypeError(f"Expected Losses value to be an in-line object, but it is currently in the '$insert' state.")
        return self.Losses


    def Losses_as(self, cls: Type[TElectricalLossesOptions])-> TElectricalLossesOptions:
        """
        Retrieves the value of Losses, ensuring it is of the specified type.
        The specified type must be one of the valid concrete types of ElectricalLosses, or the '$insert' type
        (in the case the model is defined to be inserted from an external resource).

        Useful when using type-checking development tools, as it provides a concise way to access a value with the correct type.

        Parameters
        ----------
        cls: Type[Union[ElectricalLossesInsert, LinearElectricalLosses, NonLinearElectricalLosses]]
            One of the valid concrete types of ElectricalLosses, or the '$insert' type
            (in the case the model is defined to be inserted from an external resource).

        Returns
        -------
        TElectricalLossesOptions
            A model object of the specified type.

        Raises
        ------
        TypeError
            If the value is not of the specified type.

        Examples
        --------
        Get a reference to the value when it is one of the types of ElectricalLosses:
        >>> val_obj = model_obj.Losses_as(models.LinearElectricalLosses)
        >>> val_obj = model_obj.Losses_as(models.NonLinearElectricalLosses)

        Get a reference to the value, when it was specified with a '$insert' and read in from a file:
        >>> insert_obj = model_obj.Losses_as(models.ElectricalLossesInsert)
        """
        if not isinstance(self.Losses, cls):
            raise TypeError(f"Expected Losses of type '{cls.__name__}' but was type '{type(self.Losses).__name__}'")
        return self.Losses


Generator.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.steady_calculation import SteadyCalculation
from dnv_bladed_models.steady_calculation_outputs import SteadyCalculationOutputs

from .schema_helper import SchemaHelper
from .models_impl import *


class AerodynamicInformationCalculation(SteadyCalculation):
    r"""
    Defines a calculation which produces the aerodynamic parameters of the rotor in a steady flow, including local aerodynamic loading at each blade station.  The rotor is modelled as being completely rigid in this calculation, without prebend and sweep.  The rotor is analysed in isolation without any influence from the rest of the turbine structure.
    
    Attributes
    ----------
    SteadyCalculationType : Literal['AerodynamicInformation'], default='AerodynamicInformation'
        Defines the specific type of SteadyCalculation model in use.  For a `AerodynamicInformation` object, this must always be set to a value of `AerodynamicInformation`.
    
    HubWindSpeed : float
        The wind speed at hub height.
    
    PitchAngle : float
        The pitch angle of all of the blades on the rotor or rotors.
    
    RotorSpeed : float
        The rotor speed to model during the calculation.
    
    Outputs : SteadyCalculationOutputs
    
    Notes
    -----
    
    """
    SteadyCalculationType: Literal['AerodynamicInformation'] = Field(alias="SteadyCalculationType", default='AerodynamicInformation', allow_mutation=False, const=True) # type: ignore
    HubWindSpeed: float = Field(alias="HubWindSpeed", default=None)
    PitchAngle: float = Field(alias="PitchAngle", default=None)
    RotorSpeed: float = Field(alias="RotorSpeed", default=None)
    Outputs: SteadyCalculationOutputs = Field(alias="Outputs", default=None)

    _relative_schema_path = 'SteadyCalculation/AerodynamicInformationCalculation.json'
    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        'SteadyCalculationType').merge(SteadyCalculation._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    def _entity(self) -> bool:
        return True


AerodynamicInformationCalculation.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.outputs import Outputs
from dnv_bladed_models.selected_component_output_group import SelectedComponentOutputGroup

from .schema_helper import SchemaHelper
from .models_impl import *


class TimeDomainOutputs(Outputs):
    r"""
    The definition outputs to write for this simulation.
    
    Attributes
    ----------
    TimeStepForOutputs : float
        The output time step for the simulation.
    
    LengthOfOutputBuffer : float
        The length of time to buffer the output logs.
    
    OutputSummaryInformation : bool, default=True
        If true, the summary information output group will be created.
    
    OutputExternalControllers : bool, default=True
        If true, the controller output group will be created.
    
    OutputBuoyancyInformation : bool, default=False
        If true, the buoyancy output group will be created.
    
    OutputFiniteElementMatrices : bool, default=False
        If true, the finite element output group will be created, providing far more detail about the finite element matrices.
    
    OutputSignalProperties : bool, default=False
        If true, the signal properties output group will be created.  This records the properties provided to the controller, with and without noise and other distortions.
    
    OutputWakePropagation : bool, default=False
        If true, the eddy viscosity propagation of the wake is output as a 2D table of relative velocity against radial position and distance traveled to a \".wake\" file in the output folder.
    
    OutputSoftwarePerformance : bool, default=False
        If true, the software performance output group will be created.
    
    OutputStateInformation : bool, default=False
        If true, the integrator state output group will be created.  This can be used to help understand how efficiently the integrator is coping with the simulation.
    
    OutputExternalControllerExchangeObject : bool, default=False
        If true, this will output all of the values contained in the external controller interface before and after each external controller call.  This is intended to assist debugging external controllers.
    
    OutputExternalControllerLegacySwapArray : bool, default=False
        If true, the contents of the swap array passed to a legacy controller will be logged.  This is used only when trying to debug legacy controllers, and will not produce useful results if there is more than one legacy controller being run.
    
    SelectedComponentOutputGroups : List[SelectedComponentOutputGroup], Not supported yet
        A list of references to the OutputGroup of specific components to output.  This allows the outputs of individual components to be switched off, or chosen from an available list of output regimes.  If a component is not mentioned, it will produce outputs according to its default output group, if there is one available.
    
    Notes
    -----
    
    """
    TimeStepForOutputs: float = Field(alias="TimeStepForOutputs", default=None)
    LengthOfOutputBuffer: float = Field(alias="LengthOfOutputBuffer", default=None)
    OutputSummaryInformation: bool = Field(alias="OutputSummaryInformation", default=None)
    OutputExternalControllers: bool = Field(alias="OutputExternalControllers", default=None)
    OutputBuoyancyInformation: bool = Field(alias="OutputBuoyancyInformation", default=None)
    OutputFiniteElementMatrices: bool = Field(alias="OutputFiniteElementMatrices", default=None)
    OutputSignalProperties: bool = Field(alias="OutputSignalProperties", default=None)
    OutputWakePropagation: bool = Field(alias="OutputWakePropagation", default=None)
    OutputSoftwarePerformance: bool = Field(alias="OutputSoftwarePerformance", default=None)
    OutputStateInformation: bool = Field(alias="OutputStateInformation", default=None)
    OutputExternalControllerExchangeObject: bool = Field(alias="OutputExternalControllerExchangeObject", default=None)
    OutputExternalControllerLegacySwapArray: bool = Field(alias="OutputExternalControllerLegacySwapArray", default=None)
    SelectedComponentOutputGroups: List[SelectedComponentOutputGroup] = Field(alias="SelectedComponentOutputGroups", default=list()) # Not supported yet

    _relative_schema_path = 'TimeDomainSimulation/TimeDomainOutputs/TimeDomainOutputs.json'
    _type_info = TypeInfo(
        set([]),
        set([]),
        set(['SelectedComponentOutputGroups',]),
        None).merge(Outputs._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    def _entity(self) -> bool:
        return True


TimeDomainOutputs.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.bladed_model import BladedModel

from .schema_helper import SchemaHelper
from .models_impl import *


class StructuralMode(BladedModel):
    r"""
    An object containing the DampingRatio for a particular structural mode. Used by the support structure to include structural modes and their associated DampingRatios, as well as by the blades to define WholeBladeModeDampingRatios. Users can add any metadata alongside the DampingRatio, such as _Name or _Mode. Bladed will disregard any metadata as long as it is prefixed with an underscore.
    
    Attributes
    ----------
    DampingRatio : float
        The damping ratio on the mode.  This is typically in the order of 0.005.  An underdamped mode can lead to numerical instability in the simulation, whereas overdamped mode can affect the veracity of the results.
    
    Notes
    -----
    
    """
    DampingRatio: float = Field(alias="DampingRatio", default=None)

    _relative_schema_path = 'Components/common/StructuralMode.json'
    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        None).merge(BladedModel._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    def _entity(self) -> bool:
        return True


StructuralMode.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.bladed_model import BladedModel

from .schema_helper import SchemaHelper
from .models_impl import *


class GridLossEnergySink(BladedModel):
    r"""
    An energy sink used to absorb the power from the generator in the case of a grid loss where generator braking is used.
    
    Not supported yet.
    
    Attributes
    ----------
    MaximumGeneratorTorque : float, default=0, Not supported yet
        The maximum generator torque while the energy sink is active after grid loss has occurred.  This limit is applied to generator torque demand in addition to the maximum torque demand specified in the generator model parameters.
    
    EnergyCapacityOfSink : float, default=0, Not supported yet
        The total generator mechanical input energy which can be absorbed before generator torque falls to zero following grid loss.  In a multi-rotor turbine, this is the energy capacity per rotor.
    
    Notes
    -----
    This model is not supported yet by the Bladed calculation engine.
    """
    MaximumGeneratorTorque: float = Field(alias="MaximumGeneratorTorque", default=None) # Not supported yet
    EnergyCapacityOfSink: float = Field(alias="EnergyCapacityOfSink", default=None) # Not supported yet

    _relative_schema_path = 'Turbine/ElectricalGrid/GridLossEnergySink/GridLossEnergySink.json'
    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        None).merge(BladedModel._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    def _entity(self) -> bool:
        return True


GridLossEnergySink.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.wave_diffraction_approximation import WaveDiffractionApproximation

from .schema_helper import SchemaHelper
from .models_impl import *


class MacCamyFuchs(WaveDiffractionApproximation):
    r"""
    The MacCamy-Fuchs methodology will be used, with the member diameter automatically determined from the model.
    
    Not supported yet.
    
    Attributes
    ----------
    WaveDiffractionApproximationType : Literal['MacCamyFuchs'], default='MacCamyFuchs', Not supported yet
        Defines the specific type of WaveDiffractionApproximation model in use.  For a `MacCamyFuchs` object, this must always be set to a value of `MacCamyFuchs`.
    
    MemberDiameter : float, Not supported yet
        A representative member diameter used for MacCamy-Fuchs approximation.
    
    Notes
    -----
    This model is not supported yet by the Bladed calculation engine.
    """
    WaveDiffractionApproximationType: Literal['MacCamyFuchs'] = Field(alias="WaveDiffractionApproximationType", default='MacCamyFuchs', allow_mutation=False, const=True) # Not supported yet # type: ignore
    MemberDiameter: float = Field(alias="MemberDiameter", default=None) # Not supported yet

    _relative_schema_path = 'TimeDomainSimulation/Environment/SeaState/Waves/WaveDiffractionApproximation/MacCamyFuchs.json'
    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        'WaveDiffractionApproximationType').merge(WaveDiffractionApproximation._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    def _entity(self) -> bool:
        return True


MacCamyFuchs.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.component import Component

from .schema_helper import SchemaHelper
from .models_impl import *


class Translation(Component):
    r"""
    An offset between two components, or the origin - [0,0,0] - and the first component in the Assembly tree.  This will be modelled as completely rigid.
    
    Attributes
    ----------
    ComponentType : Literal['Translation'], default='Translation'
        Defines the specific type of Component model in use.  For a `Translation` object, this must always be set to a value of `Translation`.
    
    X : float, default=0
        The translation in the X-direction of the previous component's distal frame of reference.  This will be in the global coordinate system if this is the first component in the Assembly tree.
    
    Y : float, default=0
        The translation in the Y-direction of the previous component's distal frame of reference.  This will be in the global coordinate system if this is the first component in the Assembly tree.
    
    Z : float, default=0
        The translation in the Z-direction of the previous component's distal frame of reference.  This will be in the global coordinate system if this is the first component in the Assembly tree.
    
    Notes
    -----
    
    """
    ComponentType: Literal['Translation'] = Field(alias="ComponentType", default='Translation', allow_mutation=False, const=True) # type: ignore
    X: float = Field(alias="X", default=None)
    Y: float = Field(alias="Y", default=None)
    Z: float = Field(alias="Z", default=None)

    _relative_schema_path = 'Components/Translation.json'
    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        'ComponentType').merge(Component._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    def _entity(self) -> bool:
        return True


Translation.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.response_to_demand import ResponseToDemand
class ProportionalIntegralDeriviative_DifferentialGainActionEnum(str, Enum):
    ERROR = "Error"
    FEEDBACK = "Feedback"
    SETPOINT = "Setpoint"

from .schema_helper import SchemaHelper
from .models_impl import *


class ProportionalIntegralDeriviative(ResponseToDemand):
    r"""
    Defines an proportional, integral, deriviative (PID) response to the controller's demands.
    
    Attributes
    ----------
    ResponseToDemandType : Literal['ProportionalIntegralDeriviative'], default='ProportionalIntegralDeriviative'
        Defines the specific type of ResponseToDemand model in use.  For a `ProportionalIntegralDeriviative` object, this must always be set to a value of `ProportionalIntegralDeriviative`.
    
    ProportionalGain : float, default=0
        The gain on the contemporaneous error.
    
    IntegralGain : float, default=0
        The gain on the integral from time zero till the present of the error signal.
    
    DifferentialGain : float, default=0
        The gain the filtered derivative of the error.  The derivative of the error is passed through a low-pass filter.
    
    DifferentialGainAction : ProportionalIntegralDeriviative_DifferentialGainActionEnum, default='Feedback'
        The proportional and integral terms are apllied on error.  Derivative term may also be applied on the feedback or setpoint signals.
    
    DifferentialGainTimeConstant : float, default=0
        The derivative term uses a low-pass filter on input.
    
    DesaturationTimeConstant : float, default=0
        The time constant for when the output exceeds the limits.
    
    Notes
    -----
    
    """
    ResponseToDemandType: Literal['ProportionalIntegralDeriviative'] = Field(alias="ResponseToDemandType", default='ProportionalIntegralDeriviative', allow_mutation=False, const=True) # type: ignore
    ProportionalGain: float = Field(alias="ProportionalGain", default=None)
    IntegralGain: float = Field(alias="IntegralGain", default=None)
    DifferentialGain: float = Field(alias="DifferentialGain", default=None)
    DifferentialGainAction: ProportionalIntegralDeriviative_DifferentialGainActionEnum = Field(alias="DifferentialGainAction", default=None)
    DifferentialGainTimeConstant: float = Field(alias="DifferentialGainTimeConstant", default=None)
    DesaturationTimeConstant: float = Field(alias="DesaturationTimeConstant", default=None)

    _relative_schema_path = 'Components/PitchSystem/PitchController/PitchSystemDemand/ResponseToDemand/ProportionalIntegralDeriviative.json'
    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        'ResponseToDemandType').merge(ResponseToDemand._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    def _entity(self) -> bool:
        return True


ProportionalIntegralDeriviative.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.tower_can_cross_section_properties_where_different import TowerCanCrossSectionPropertiesWhereDifferent
from dnv_bladed_models.tower_can_torsional_properties import TowerCanTorsionalProperties

from .schema_helper import SchemaHelper
from .models_impl import *


class TowerCanPropertiesExplicitWhereDifferent(TowerCanCrossSectionPropertiesWhereDifferent):
    r"""
    The definition of a single tower can cross-section where any structural properties can be explicitly specified if they are different to those specified in the BaseCrossSection.
    
    Attributes
    ----------
    MassPerUnitLength : float
        The mass per unit length. If omitted, the value from the BaseCrossSection will be used.
    
    BendingStiffness : float
        The bending stiffness of the cross-section. If omitted, the value from the BaseCrossSection will be used.
    
    ShearStiffness : float
        The shear stiffness or shear modulus of the cross-section. If omitted, the value from the BaseCrossSection will be used.
    
    TorsionalProperties : TowerCanTorsionalProperties
    
    Notes
    -----
    
    """
    MassPerUnitLength: float = Field(alias="MassPerUnitLength", default=None)
    BendingStiffness: float = Field(alias="BendingStiffness", default=None)
    ShearStiffness: float = Field(alias="ShearStiffness", default=None)
    TorsionalProperties: TowerCanTorsionalProperties = Field(alias="TorsionalProperties", default=None)

    _relative_schema_path = 'Components/Tower/TowerCan/TowerCanPropertiesExplicitWhereDifferent/TowerCanPropertiesExplicitWhereDifferent.json'
    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        None).merge(TowerCanCrossSectionPropertiesWhereDifferent._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    def _entity(self) -> bool:
        return True


TowerCanPropertiesExplicitWhereDifferent.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.bladed_model import BladedModel

from .schema_helper import SchemaHelper
from .models_impl import *


class LowSpeedShaft(BladedModel):
    r"""
    The low speed shaft bending definition.
    
    Attributes
    ----------
    EffectiveShaftLength : float
        The length of the low speed shaft to be considered for the bending model, beginning at the outside face of the main bearing.
    
    HingePosition : float
        The fraction of the low speed shaft length (from the hub centre) where the hinge between the two massless rigid shafts are connected.
    
    TorsionalStiffness : float
        The shaft stiffness about the axis of rotation.
    
    TorsionalDamping : float
        The shaft damping about the axis of rotation.
    
    BendingStiffness : float
        The shaft's bending stiffness about any axis perpendicular to the axis of rotation.
    
    BendingDamping : float
        The shaft's bending damping about any axis perpendicular to the axis of rotation.
    
    Notes
    -----
    
    """
    EffectiveShaftLength: float = Field(alias="EffectiveShaftLength", default=None)
    HingePosition: float = Field(alias="HingePosition", default=None)
    TorsionalStiffness: float = Field(alias="TorsionalStiffness", default=None)
    TorsionalDamping: float = Field(alias="TorsionalDamping", default=None)
    BendingStiffness: float = Field(alias="BendingStiffness", default=None)
    BendingDamping: float = Field(alias="BendingDamping", default=None)

    _relative_schema_path = 'Components/DrivetrainAndNacelle/LowSpeedShaft/LowSpeedShaft.json'
    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        None).merge(BladedModel._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    def _entity(self) -> bool:
        return True


LowSpeedShaft.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.structural_mode import StructuralMode
from dnv_bladed_models.structural_modelling import StructuralModelling

from .schema_helper import SchemaHelper
from .models_impl import *


class ModalStructuralModelling(StructuralModelling):
    r"""
    The properties for the structural modelling using modal reduction.
    
    Attributes
    ----------
    StructuralModellingType : Literal['ModalStructuralModelling'], default='ModalStructuralModelling'
        Defines the specific type of StructuralModelling model in use.  For a `ModalStructuralModelling` object, this must always be set to a value of `ModalStructuralModelling`.
    
    StructuralModes : List[StructuralMode]
        List of included modes and their associated damping ratios. The presence of any of these objects in the StructuralModes list indicates that the mode should be calculated, even if it does not include a DampingRatio. If no modes are provided, the structure will be considered rigid. Users can add any metadata alongside the DampingRatio, such as _Name or _Mode. Bladed will disregard any metadata as long as it is prefixed with an underscore.
    
    Notes
    -----
    
    """
    StructuralModellingType: Literal['ModalStructuralModelling'] = Field(alias="StructuralModellingType", default='ModalStructuralModelling', allow_mutation=False, const=True) # type: ignore
    StructuralModes: List[StructuralMode] = Field(alias="StructuralModes", default=list())

    _relative_schema_path = 'Components/StructuralModelling/ModalStructuralModelling.json'
    _type_info = TypeInfo(
        set([]),
        set([]),
        set(['StructuralModes',]),
        'StructuralModellingType').merge(StructuralModelling._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    def _entity(self) -> bool:
        return True


ModalStructuralModelling.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.bladed_model import BladedModel
from dnv_bladed_models.nominal_hub_wind_speed_vs_rotor_speed_and_pitch_angle import NominalHubWindSpeedVsRotorSpeedAndPitchAngle
from dnv_bladed_models.variable_speed_pitch_regulated_control_model import VariableSpeedPitchRegulatedControlModel

from .schema_helper import SchemaHelper
from .models_impl import *


class TurbineOperationalParameters(BladedModel):
    r"""
    The general operational parameters for the turbine. These are used to calculate steady state conditions for steady calculation types or initial conditions for time-domain calculation types.
    
    Attributes
    ----------
    NominalHubWindSpeedVsRotorSpeedAndPitchAngle : List[NominalHubWindSpeedVsRotorSpeedAndPitchAngle]
        User defined look-up table to define the steady state conditions of rotor speed and pitch angle over wind speed. If defined Bladed will use these values instead of trying to determine them.
    
    VariableSpeedPitchRegulatedControlModel : VariableSpeedPitchRegulatedControlModel
    
    PitchAngleWhilstIdling : float, default=1.570796326794
        The pitch angle of the blades when the turbine is idling.  The default is 90 degrees.
    
    PitchAngleWhilstParked : float, default=1.570796326794
        The pitch angle of the blades when the turbine is parked.  The default is 90 degrees.
    
    Notes
    -----
    
    """
    NominalHubWindSpeedVsRotorSpeedAndPitchAngle: List[NominalHubWindSpeedVsRotorSpeedAndPitchAngle] = Field(alias="NominalHubWindSpeedVsRotorSpeedAndPitchAngle", default=list())
    VariableSpeedPitchRegulatedControlModel: VariableSpeedPitchRegulatedControlModel = Field(alias="VariableSpeedPitchRegulatedControlModel", default=None)
    PitchAngleWhilstIdling: float = Field(alias="PitchAngleWhilstIdling", default=None)
    PitchAngleWhilstParked: float = Field(alias="PitchAngleWhilstParked", default=None)

    _relative_schema_path = 'Turbine/TurbineOperationalParameters/TurbineOperationalParameters.json'
    _type_info = TypeInfo(
        set([]),
        set([]),
        set(['NominalHubWindSpeedVsRotorSpeedAndPitchAngle',]),
        None).merge(BladedModel._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    def _entity(self) -> bool:
        return True


TurbineOperationalParameters.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.dynamic_wake import DynamicWake

from .schema_helper import SchemaHelper
from .models_impl import *


class EquilibriumWakeModel(DynamicWake):
    r"""
    The equilibrium wake model instantaneously updates the induction in response to changes operating condition.  This represents the steady state solution.  It should not be used in linearisation as it would represent an instantaneous response of the wake, where in reality the response is significantly lagged.  It is also not a good approach for time domain as the lagged wake response is not captured.
    
    Attributes
    ----------
    DynamicWakeType : Literal['EquilibriumWakeModel'], default='EquilibriumWakeModel'
        Defines the specific type of DynamicWake model in use.  For a `EquilibriumWakeModel` object, this must always be set to a value of `EquilibriumWakeModel`.
    
    Notes
    -----
    
    """
    DynamicWakeType: Literal['EquilibriumWakeModel'] = Field(alias="DynamicWakeType", default='EquilibriumWakeModel', allow_mutation=False, const=True) # type: ignore

    _relative_schema_path = 'Settings/AerodynamicSettings/AerodynamicModel/MomentumTheoryCorrections/DynamicWake/EquilibriumWakeModel.json'
    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        'DynamicWakeType').merge(DynamicWake._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    def _entity(self) -> bool:
        return True


EquilibriumWakeModel.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from abc import ABC

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.steady_calculation import SteadyCalculation
from dnv_bladed_models.steady_calculation_outputs import SteadyCalculationOutputs

from .schema_helper import SchemaHelper
from .models_impl import *


class LinearisationCalculation(SteadyCalculation, ABC):
    r"""
    The common properties of calculations which use small perturbations to generate system responses, from which the dynamics of the fully coupled system can be analysed.
    
    Not supported yet.
    
    Attributes
    ----------
    MinimumCorrelationCoefficient : float, default=0.8, Not supported yet
        The minimum acceptable correlation coefficient for the relationship between a state value and a state derivative.  If it is below the minimum, the relationship is disregarded and a zero value is taken.
    
    Outputs : SteadyCalculationOutputs
    
    TransformRotorModesToNonRotating : bool, default=False, Not supported yet
        If true, a multi-blade coordinate (MBC) transform will be performed to transform the rotating modes into the stationary frame of reference.  This will generate forward and backward whirling modes in a Campbell diagram.
    
    AlignWindFieldWithDeflectedHubAxis : bool, default=True, Not supported yet
        If true, the deflected steady-state operating conditions of the turbine are calculated, and then the wind field is rotated to align with the rotating axis of the (first) hub for linearisation.
    
    Notes
    -----
    This model is not supported yet by the Bladed calculation engine.
    """
    MinimumCorrelationCoefficient: float = Field(alias="MinimumCorrelationCoefficient", default=None) # Not supported yet
    Outputs: SteadyCalculationOutputs = Field(alias="Outputs", default=None)
    TransformRotorModesToNonRotating: bool = Field(alias="TransformRotorModesToNonRotating", default=None) # Not supported yet
    AlignWindFieldWithDeflectedHubAxis: bool = Field(alias="AlignWindFieldWithDeflectedHubAxis", default=None) # Not supported yet

    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        None).merge(SteadyCalculation._type_info)


LinearisationCalculation.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.bladed_model import BladedModel

from .schema_helper import SchemaHelper
from .models_impl import *


class ExternalModule(BladedModel):
    r"""
    A specialised External Module component.  This is in the form of an external library (a dynamic link library on Windows, or a shared object on Linux).  This acts as a component in the Bladed assembly, and creates multibody objects to attach to the previous component.
    
    Attributes
    ----------
    Filepath : str
        The absolute filepath to the dynamic linked library (Windows) or the shared object (Linux).
    
    ParametersAsString : str
        A string that will be passed to the external module.
    
    ParametersAsJson : Dict[str, Any]
        A JSON object that will be serialised as a string and passed to the external module.
    
    Notes
    -----
    
    """
    Filepath: str = Field(alias="Filepath", default=None)
    ParametersAsString: str = Field(alias="ParametersAsString", default=None)
    ParametersAsJson: Dict[str, Any] = Field(alias="ParametersAsJson", default=None)

    _relative_schema_path = 'common/ExternalModule.json'
    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        None).merge(BladedModel._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    def _entity(self) -> bool:
        return True


ExternalModule.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from abc import ABC

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder

from .schema_helper import SchemaHelper
from .models_impl import *
TBladedModel = TypeVar('TBladedModel', bound='BladedModel')


class BladedModel(BladedEntity, ABC):
    r"""
    The base schema for all Bladed schema objects.
    
    Attributes
    ----------
    Schema : str
        The location of the JSON schema used to validate this object and enable enhanced tooling support.
    
    insert : str
        A path to a resource from which a valid JSON model object can be resolved. All properties will be taken from the resolved object; no properties can be specified in-line.
    
    Notes
    -----
    
    """
    Schema: str = Field(alias="$schema", default=None)
    insert: str = Field(alias="$insert", default=None)

    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        None).merge(BladedEntity._type_info)


    def extract_to_insert_from_file(
            self, 
            insert_path: Union[str, Path], 
            relative_to: Optional[Union[str, Path]] = None,
            write_file: bool = True) -> Path:
        r"""
        Sets this model to be inserted from a file when read by the Bladed application, and optionally writes the model to the new file. 
        Subsequently, when the parent model is serialized to JSON, only the '$insert' field will then be rendered for this model object.

        Parameters
        ----------
        insert_path : str or Path
            The seed for the '$insert' value for the JSON file that will contain the model document. 
            This can be an absolute path or a relative path.
            If absolute, and relative_to is supplied, a relative path will be calculated and set as the '$insert' value.
            If it is a relative path, and no relative_to is supplied, it will be interpreted relative to the current working directory.
        
        relative_to : str or Path
            The path to the file or directory that will contain the parent or owning model document.
            If supplied, and insert_path is absolute, it is used to calculate a relative path value, which is then set as the $insert value.
            If it is a relative path, it will be relative to the current working directory.

        write_file : bool = True
            If True, a JSON file will be written to the calculated file location, and contain the JSON model for insertion.

        Returns
        -------
        Path
            The full path of the file that the extracted model will be written to.

        Raises
        ------
        ValueError
            If the insert_path is None or empty.
            If a relative_to is provided and it is not a valid sub-path.
            If the model is already set to insert, and write_file is True.

        Examples
        --------
        1. Set the model to insert from a relative directory, using a relative seed path:
           - writes a new file to `c:/absolute/path/sub-dir/ChildModel.json` containing the child model JSON document
           - sets the insert path to `sub-dir/ChildModel.json` on the parent property object
           - writes the parent model to file, now containing the 'insert' reference to the child model

        >>> child_model_path = ParentModel.ChildModel.extract_to_insert_from_file("sub-dir/ChildModel.json", relative_to="c:/absolute/path/ParentModel.json")
        >>> ParentModel.to_file("c:/absolute/path/ParentModel.json")

        2. Set the model to insert from a relative directory, using an absolute seed path:
           - writes a new file to 'c:/absolute/path/sub-dir/ChildModel.json' containing the child model JSON document
           - sets the insert path to `../sub-dir/ChildModel.json` on the parent property object
           - writes the parent model to file, now containing the 'insert' reference to the child model

        >>> ParentModel.ChildModel.extract_to_insert_from_file("c:/absolute/path/sub-dir/ChildModel.json", relative_to="c:/absolute/path/parent-dir/ParentModel.json")
        >>> ParentModel.to_file("c:/absolute/path/parent-dir/ParentModel.json")
        """
        if insert_path is None or insert_path == "": # type: ignore
            raise ValueError("insert_path is required")
        
        ip: Path = Path(insert_path) if not isinstance(insert_path, Path) else insert_path

        insert_loc: Path = ip

        # Calculate the relative path to the relative_to if required
        if relative_to is not None and str(relative_to) != "" and ip.is_absolute():
            relp: Path = Path(relative_to).resolve() if not isinstance(relative_to, Path) else relative_to.resolve()
            relp_d: Path = relp if relp.is_dir() else relp.parent
            insert_loc = Path(os.path.relpath(ip.resolve(), relp_d))
            
        if relative_to is not None and str(relative_to) != "" and not ip.is_absolute():
            relp: Path = Path(relative_to) if not isinstance(relative_to, Path) else relative_to
            relp_d: Path = relp if relp.is_dir() else relp.parent
            file_loc = relp_d.joinpath(insert_path).resolve()
        else:
            file_loc = ip if ip.is_absolute() else ip.resolve()
            
        if write_file:
            if self.is_insert:
                raise ValueError("Cannot extract a model to a new file when it has already been set as an insert.")
            
            if not file_loc.parent.exists():
                file_loc.parent.mkdir(parents=True, exist_ok=True)

            self.to_file(file_loc)
        
        # forward slashes render a little neater in JSON as they don't require escaping.
        # there's no functional difference either way, and the Bladed application will accept either.
        self.insert = insert_loc.as_posix()
        return file_loc



    def to_json(self, indent: Optional[int] = 2, **json_kwargs: Any) -> str:
        r"""
        Generates a JSON string representation of the model.

        Parameters
        ----------
        indent : int
            The whitespace indentation to use for formatting, as per json.dumps().

        Examples
        --------
        >>> model.to_json()
        Renders the full JSON representation of the model object.
        """

        json_kwargs['by_alias'] = True
        json_kwargs['exclude_unset'] = False
        json_kwargs['exclude_none'] = True
        if self.Schema is None:
            self.Schema = SchemaHelper.construct_schema_url(getattr(self, '_relative_schema_path'))
        
        return super().json(indent=indent, **json_kwargs)


    @classmethod
    def from_file(cls: Type[TBladedModel], path: Union[str, Path]) -> TBladedModel:
        r"""
        Loads a model from a given file path.

        Parameters
        ----------
        path : string
            The file path to the model.

        Returns
        -------
        
            The model object.

        Raises
        ------
        ValueError, ValidationError
            If the JSON document does not correctly describe the model according to the model schema.

        Examples
        --------
        >>> model = ModelClass.from_file('/path/to/file')
        """
        
        return super().parse_file(path=path)


    @classmethod
    def from_json(cls: Type[TBladedModel], b: StrBytes) -> TBladedModel:
        r"""
        Creates a model object from a JSON string.

        Parameters
        ----------
        b: StrBytes
            The JSON string describing the model.

        Returns
        -------
        
            The model object.

        Raises
        ------
        ValueError, ValidationError
            If the JSON document does not correctly describe the model according to the model schema.

        Examples
        --------
        >>> model = ModelClass.from_json('{ ... }')
        """

        return super().parse_raw(
            b=b,
            content_type='application/json')
        

    @classmethod
    def from_dict(cls: Type[TBladedModel], obj: Any) -> TBladedModel:
        r"""
        Creates a model object from a dict.
        
        Parameters
        ----------
        obj : Any
            The dictionary object describing the model.

        Returns
        -------
        
            The model object.

        Raises
        ------
        ValueError, ValidationError
            If the JSON document does not correctly describe the model according to the model schema.

        Examples
        --------
        >>> model = ModelClass.from_dict({ ... })
        """
        
        return cls.parse_obj(obj=obj)


    def to_file(self, path: Union[str, Path]) -> None:
        r"""
        Writes the model as a JSON document to a file with UTF8 encoding.

        Parameters
        ----------                
        path : string
            The file path to which the model will be written.

        Examples
        --------
        >>> model.to_file('/path/to/file')
        """

        with open(file=path, mode='w', encoding="utf8") as output_file:
            output_file.write(self.to_json())


BladedModel.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.evolving_turbulence import EvolvingTurbulence
from dnv_bladed_models.evolving_turbulence_exponential import EvolvingTurbulenceExponential
from dnv_bladed_models.evolving_turbulence_insert import EvolvingTurbulenceInsert
from dnv_bladed_models.evolving_turbulence_kristensen import EvolvingTurbulenceKristensen
from dnv_bladed_models.time_domain_wind import TimeDomainWind
class TurbulentWind_CentreTurbulenceFileOnEnum(str, Enum):
    CENTRED_ON_HUB = "CENTRED_ON_HUB"
    BEST_FIT = "BEST_FIT"
class TurbulentWind_InterpolationMethodEnum(str, Enum):
    LINEAR = "LINEAR"
    CUBIC_IN_ROTOR_PLANE = "CUBIC_IN_ROTOR_PLANE"
    ALL_CUBIC = "ALL_CUBIC"

from .schema_helper import SchemaHelper
from .models_impl import *

TEvolvingTurbulenceOptions = TypeVar('TEvolvingTurbulenceOptions', EvolvingTurbulenceExponential, EvolvingTurbulenceKristensen, EvolvingTurbulenceInsert, EvolvingTurbulence, )

class TurbulentWind(TimeDomainWind):
    r"""
    The definition of a turbulent flow field, with the values for the turbulence defined in an external file.
    
    Attributes
    ----------
    WindType : Literal['TurbulentWind'], default='TurbulentWind'
        Defines the specific type of Wind model in use.  For a `TurbulentWind` object, this must always be set to a value of `TurbulentWind`.
    
    MeanSpeed : float
        The mean wind speed upon which the turbulence will be added.  This must correspond with the mean wind speed used to create the turbulence file.
    
    TurbulenceFilepath : str
        The filepath or URI of the turbulence file.
    
    TurbulenceIntensity : float
        The turbulence intensity in the longitudinal (global X) direction.  This is used to scale the turbulence provided in the file.
    
    TurbulenceIntensityLateral : float
        The turbulence intensity in the lateral (global Y) direction.  This is typically in the order of 80% of the longitudinal turbulence intensity.
    
    TurbulenceIntensityVertical : float
        The turbulence intensity in the vertical (global Z) direction.  This is typically in the order of 50% of the longitudinal turbulence intensity.
    
    CentreTurbulenceFileOn : TurbulentWind_CentreTurbulenceFileOnEnum
        The method used to position the data in the turbulence file relative to the turbine.  If any part of the rotor exceeds this box, the simulation will terminate with an exception.
    
    CentreTurbulenceFileAtHeight : float
        The height at which to centre the data in the turbulence file.  If any part of the rotor exceeds this box, the simulation will terminate with an exception.
    
    InterpolationMethod : TurbulentWind_InterpolationMethodEnum, default='ALL_CUBIC'
        The method used to interpolate the wind speed in between the provided turbulence locations.
    
    RepeatTurbulenceFile : bool, default=False
        If true, the turbulence file will be \"looped\".  If false, the turbulence will be 0 in all three components once the end of the file has been reached.  Using part of a turbulence file may invalidate its turbulence statistics, and no effort is made by Bladed to ensure coherence at the point when it transitions from the end of the wind file back to the beginning.
    
    EvolvingTurbulence : Union[EvolvingTurbulenceExponential, EvolvingTurbulenceKristensen, EvolvingTurbulenceInsert]
    
    TurbulenceFileStartTime : float, default=0
        The time into turbulent wind file at start of simulation.  This can be used to synchronise the wind file with simulation.
    
    DirectionShear : float, default=0
        The direction shear, otherwise known as \"wind veer\".  This models the case where the direction of the wind field varies as the height increases.
    
    Notes
    -----
    
    """
    WindType: Literal['TurbulentWind'] = Field(alias="WindType", default='TurbulentWind', allow_mutation=False, const=True) # type: ignore
    MeanSpeed: float = Field(alias="MeanSpeed", default=None)
    TurbulenceFilepath: str = Field(alias="TurbulenceFilepath", default=None)
    TurbulenceIntensity: float = Field(alias="TurbulenceIntensity", default=None)
    TurbulenceIntensityLateral: float = Field(alias="TurbulenceIntensityLateral", default=None)
    TurbulenceIntensityVertical: float = Field(alias="TurbulenceIntensityVertical", default=None)
    CentreTurbulenceFileOn: TurbulentWind_CentreTurbulenceFileOnEnum = Field(alias="CentreTurbulenceFileOn", default=None)
    CentreTurbulenceFileAtHeight: float = Field(alias="CentreTurbulenceFileAtHeight", default=None)
    InterpolationMethod: TurbulentWind_InterpolationMethodEnum = Field(alias="InterpolationMethod", default=None)
    RepeatTurbulenceFile: bool = Field(alias="RepeatTurbulenceFile", default=None)
    EvolvingTurbulence: Union[EvolvingTurbulenceExponential, EvolvingTurbulenceKristensen, EvolvingTurbulenceInsert] = Field(alias="EvolvingTurbulence", default=None, discriminator='EvolvingTurbulenceType')
    TurbulenceFileStartTime: float = Field(alias="TurbulenceFileStartTime", default=None)
    DirectionShear: float = Field(alias="DirectionShear", default=None)

    _relative_schema_path = 'TimeDomainSimulation/Environment/Wind/TurbulentWind.json'
    _type_info = TypeInfo(
        set([('EvolvingTurbulence', 'EvolvingTurbulenceType'),]),
        set([]),
        set([]),
        'WindType').merge(TimeDomainWind._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    @property
    def EvolvingTurbulence_as_EvolvingTurbulenceExponential(self) -> EvolvingTurbulenceExponential:
        """
        Retrieves the value of EvolvingTurbulence guaranteeing it is a EvolvingTurbulenceExponential; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        EvolvingTurbulenceExponential
            A model object, guaranteed to be a EvolvingTurbulenceExponential.

        Raises
        ------
        TypeError
            If the value is not a EvolvingTurbulenceExponential.
        """
        return self.EvolvingTurbulence_as(EvolvingTurbulenceExponential)


    @property
    def EvolvingTurbulence_as_EvolvingTurbulenceKristensen(self) -> EvolvingTurbulenceKristensen:
        """
        Retrieves the value of EvolvingTurbulence guaranteeing it is a EvolvingTurbulenceKristensen; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        EvolvingTurbulenceKristensen
            A model object, guaranteed to be a EvolvingTurbulenceKristensen.

        Raises
        ------
        TypeError
            If the value is not a EvolvingTurbulenceKristensen.
        """
        return self.EvolvingTurbulence_as(EvolvingTurbulenceKristensen)


    @property
    def EvolvingTurbulence_as_inline(self) -> Union[EvolvingTurbulenceExponential, EvolvingTurbulenceKristensen]:
        """
        Retrieves the value of EvolvingTurbulence as a model object; if the value is specified with a '$insert', an error is raised.

        Returns
        -------
        Union[EvolvingTurbulenceExponential, EvolvingTurbulenceKristensen]
            A model object at the specified index, guaranteed to not be a '$insert'.

        Raises
        ------
        TypeError
            If the value is not one of the concrete types of EvolvingTurbulence; i.e. it is specified with a '$insert'.
        """
        if isinstance(self.EvolvingTurbulence, EvolvingTurbulenceInsert) or self.EvolvingTurbulence.is_insert:
            raise TypeError(f"Expected EvolvingTurbulence value to be an in-line object, but it is currently in the '$insert' state.")
        return self.EvolvingTurbulence


    def EvolvingTurbulence_as(self, cls: Type[TEvolvingTurbulenceOptions])-> TEvolvingTurbulenceOptions:
        """
        Retrieves the value of EvolvingTurbulence, ensuring it is of the specified type.
        The specified type must be one of the valid concrete types of EvolvingTurbulence, or the '$insert' type
        (in the case the model is defined to be inserted from an external resource).

        Useful when using type-checking development tools, as it provides a concise way to access a value with the correct type.

        Parameters
        ----------
        cls: Type[Union[EvolvingTurbulenceExponential, EvolvingTurbulenceKristensen, EvolvingTurbulenceInsert]]
            One of the valid concrete types of EvolvingTurbulence, or the '$insert' type
            (in the case the model is defined to be inserted from an external resource).

        Returns
        -------
        TEvolvingTurbulenceOptions
            A model object of the specified type.

        Raises
        ------
        TypeError
            If the value is not of the specified type.

        Examples
        --------
        Get a reference to the value when it is one of the types of EvolvingTurbulence:
        >>> val_obj = model_obj.EvolvingTurbulence_as(models.EvolvingTurbulenceExponential)
        >>> val_obj = model_obj.EvolvingTurbulence_as(models.EvolvingTurbulenceKristensen)

        Get a reference to the value, when it was specified with a '$insert' and read in from a file:
        >>> insert_obj = model_obj.EvolvingTurbulence_as(models.EvolvingTurbulenceInsert)
        """
        if not isinstance(self.EvolvingTurbulence, cls):
            raise TypeError(f"Expected EvolvingTurbulence of type '{cls.__name__}' but was type '{type(self.EvolvingTurbulence).__name__}'")
        return self.EvolvingTurbulence


    def _entity(self) -> bool:
        return True


TurbulentWind.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.bladed_model import BladedModel

from .schema_helper import SchemaHelper
from .models_impl import *


class TowerCanModelling(BladedModel):
    r"""
    Parameters controlling the way the can will be modelled by Bladed.
    
    Attributes
    ----------
    MaximumNodeSpacing : float
        If any two nodes are further spaced apart than this, an additional node or nodes will be added equally spaced between them.  The default is infinity, meaning that no extra nodes will ever be added unless NumberOfIntermediateNodes has been specified.
    
    NumberOfIntermediateNodes : int, default=0
        The number of structural nodes to be added, equally spaced between the top and bottom of the can.  The default is 0, meaning that (unless the MaximumNodeSpacing requires additional nodes to be added) the can will be represented by a single beam.
    
    Notes
    -----
    
    """
    MaximumNodeSpacing: float = Field(alias="MaximumNodeSpacing", default=None)
    NumberOfIntermediateNodes: int = Field(alias="NumberOfIntermediateNodes", default=None)

    _relative_schema_path = 'Components/Tower/TowerCan/TowerCanModelling/TowerCanModelling.json'
    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        None).merge(BladedModel._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    def _entity(self) -> bool:
        return True


TowerCanModelling.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.initial_position import InitialPosition

from .schema_helper import SchemaHelper
from .models_impl import *


class InitialYawAngle(InitialPosition):
    r"""
    The initial position of the yaw system.
    
    Attributes
    ----------
    InitialConditionType : Literal['InitialYawAngle'], default='InitialYawAngle'
        Defines the specific type of InitialCondition model in use.  For a `InitialYawAngle` object, this must always be set to a value of `InitialYawAngle`.
    
    YawAngle : float
        The yaw angle of the yaw system.
    
    MaintainInitialValueThroughoutSimulation : bool, default=False
        If true, the yaw system will be prescribed to maintain whatever yaw angle was determined during initial conditions.  This cannot be overcome by aerodynamic loads or other physical forces.
    
    Notes
    -----
    
    """
    InitialConditionType: Literal['InitialYawAngle'] = Field(alias="InitialConditionType", default='InitialYawAngle', allow_mutation=False, const=True) # type: ignore
    YawAngle: float = Field(alias="YawAngle", default=None)
    MaintainInitialValueThroughoutSimulation: bool = Field(alias="MaintainInitialValueThroughoutSimulation", default=None)

    _relative_schema_path = 'TimeDomainSimulation/InitialCondition/InitialYawAngle.json'
    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        'InitialConditionType').merge(InitialPosition._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    def _entity(self) -> bool:
        return True


InitialYawAngle.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.bladed_model import BladedModel

from .schema_helper import SchemaHelper
from .models_impl import *


class TowerShadowPowles(BladedModel):
    r"""
    Parameters for the empirical Powles tower shadow model. This implementation scales the wake width in proportion to the square root of the distance downstream.
    
    Attributes
    ----------
    ShadowWidthProportion : float, default=0
        The wake width as a proportion of the local obstruction's diameter.  The wake width is constant in the case of there being no downstream variation, but in the case of an inverse square variation, the wake width is applied at the reference length, \"Powles L Reference\" (the width at other points scaling in proporton to the square root of the distance downstream).
    
    MaximumDeficit : float, default=0
        The maximum deficit for use in the empirical Powles equation, expressed as a fraction of the unperturbed flow.
    
    LengthReference : float, default=0
        The distance downstream of the obstruction that the maximum deficit is applicable for, expressed as a multiple of the obstruction's diameter.
    
    Notes
    -----
    
    """
    ShadowWidthProportion: float = Field(alias="ShadowWidthProportion", default=None)
    MaximumDeficit: float = Field(alias="MaximumDeficit", default=None)
    LengthReference: float = Field(alias="LengthReference", default=None)

    _relative_schema_path = 'Components/Tower/TowerAerodynamicProperties/FlowObstruction/TowerShadowPowles/TowerShadowPowles.json'
    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        None).merge(BladedModel._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    def _entity(self) -> bool:
        return True


TowerShadowPowles.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.bladed_model import BladedModel
from dnv_bladed_models.meandering_wake import MeanderingWake
from dnv_bladed_models.position_of_upwind_turbine import PositionOfUpwindTurbine
from dnv_bladed_models.wake_properties_of_upstream_turbine import WakePropertiesOfUpstreamTurbine

from .schema_helper import SchemaHelper
from .models_impl import *


class DynamicUpstreamWake(BladedModel):
    r"""
    
    
    Attributes
    ----------
    RadialStepSize : float, default=0.05
        The resolution of the wake deficit profile in the radial direction.
    
    NumberOfRadialPoints : int, default=50
        The total number of points in the radial wake deficit profile.
    
    StreamwiseStep : float, default=0.1
        The integration step in streamwise direction for the propagation of the wake.
    
    MixingLength : float, default=1.323
        The length scale of the part of ambient turbulence which affects the wake deficit evolution.
    
    ShearCalibrationConstant : float, default=0.008
        The calibration factor for self-generated turbulence.  IEC edition 4 recommends a value of 0.008.
    
    AmbientCalibrationConstant : float, default=0.023
        The calibration factor for influence of ambient turbulence.  IEC edition 4 recommends a value of 0.023.
    
    PositionOfUpwindTurbine : PositionOfUpwindTurbine
    
    WakePropertiesOfUpstreamTurbine : WakePropertiesOfUpstreamTurbine
    
    MeanderingWake : MeanderingWake
    
    Notes
    -----
    
    """
    RadialStepSize: float = Field(alias="RadialStepSize", default=None)
    NumberOfRadialPoints: int = Field(alias="NumberOfRadialPoints", default=None)
    StreamwiseStep: float = Field(alias="StreamwiseStep", default=None)
    MixingLength: float = Field(alias="MixingLength", default=None)
    ShearCalibrationConstant: float = Field(alias="ShearCalibrationConstant", default=None)
    AmbientCalibrationConstant: float = Field(alias="AmbientCalibrationConstant", default=None)
    PositionOfUpwindTurbine: PositionOfUpwindTurbine = Field(alias="PositionOfUpwindTurbine", default=None)
    WakePropertiesOfUpstreamTurbine: WakePropertiesOfUpstreamTurbine = Field(alias="WakePropertiesOfUpstreamTurbine", default=None)
    MeanderingWake: MeanderingWake = Field(alias="MeanderingWake", default=None)

    _relative_schema_path = 'TimeDomainSimulation/Environment/Wind/DynamicUpstreamWake/DynamicUpstreamWake.json'
    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        None).merge(BladedModel._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    def _entity(self) -> bool:
        return True


DynamicUpstreamWake.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.aerofoil_library import AerofoilLibrary
from dnv_bladed_models.aileron_aerofoil_library import AileronAerofoilLibrary
from dnv_bladed_models.blade_additional_inertia import BladeAdditionalInertia
from dnv_bladed_models.blade_modelling import BladeModelling
from dnv_bladed_models.blade_modelling_insert import BladeModellingInsert
from dnv_bladed_models.blade_output_group_library import BladeOutputGroupLibrary
from dnv_bladed_models.component import Component
from dnv_bladed_models.cross_section import CrossSection
from dnv_bladed_models.finite_element_blade_modelling import FiniteElementBladeModelling
from dnv_bladed_models.interpolated_aerofoil_library import InterpolatedAerofoilLibrary
from dnv_bladed_models.modal_blade_modelling import ModalBladeModelling
from dnv_bladed_models.mounting import Mounting
from dnv_bladed_models.rigid_blade_modelling import RigidBladeModelling

from .schema_helper import SchemaHelper
from .models_impl import *

TBladeModellingOptions = TypeVar('TBladeModellingOptions', FiniteElementBladeModelling, BladeModellingInsert, ModalBladeModelling, RigidBladeModelling, BladeModelling, )

class Blade(Component):
    r"""
    A blade component.
    
    Attributes
    ----------
    ComponentType : Literal['Blade'], default='Blade'
        Defines the specific type of Component model in use.  For a `Blade` object, this must always be set to a value of `Blade`.
    
    Modelling : Union[FiniteElementBladeModelling, BladeModellingInsert, ModalBladeModelling, RigidBladeModelling]
    
    AerofoilLibrary : AerofoilLibrary
    
    InterpolatedAerofoilLibrary : InterpolatedAerofoilLibrary
    
    AileronAerofoilLibrary : AileronAerofoilLibrary, Not supported yet
    
    Mounting : Mounting
    
    ToleranceForRepeatedCrossSections : float, default=0.001
        The tolerance used to determine whether two blade cross-sections are merely adjacent, or represent a step-change in properties at a discrete point.  If the plane point of the reference axes lie within this distance of each other, then it will be taken that the two blade cross-section definitions represent the properties inboard and outboard of a single structural point.
    
    CrossSections : List[CrossSection]
        A list of blade cross-sections which describes the geometric, structural, and aerodynamic properties of the blade.
    
    AdditionalInertia : BladeAdditionalInertia
    
    OutputGroups : BladeOutputGroupLibrary, Not supported yet
    
    Notes
    -----
    
    """
    ComponentType: Literal['Blade'] = Field(alias="ComponentType", default='Blade', allow_mutation=False, const=True) # type: ignore
    Modelling: Union[FiniteElementBladeModelling, BladeModellingInsert, ModalBladeModelling, RigidBladeModelling] = Field(alias="Modelling", default=None, discriminator='BladeModellingType')
    AerofoilLibrary: AerofoilLibrary = Field(alias="AerofoilLibrary", default=AerofoilLibrary())
    InterpolatedAerofoilLibrary: InterpolatedAerofoilLibrary = Field(alias="InterpolatedAerofoilLibrary", default=InterpolatedAerofoilLibrary())
    AileronAerofoilLibrary: AileronAerofoilLibrary = Field(alias="AileronAerofoilLibrary", default=AileronAerofoilLibrary()) # Not supported yet
    Mounting: Mounting = Field(alias="Mounting", default=None)
    ToleranceForRepeatedCrossSections: float = Field(alias="ToleranceForRepeatedCrossSections", default=None)
    CrossSections: List[CrossSection] = Field(alias="CrossSections", default=list())
    AdditionalInertia: BladeAdditionalInertia = Field(alias="AdditionalInertia", default=None)
    OutputGroups: BladeOutputGroupLibrary = Field(alias="OutputGroups", default=BladeOutputGroupLibrary()) # Not supported yet

    _relative_schema_path = 'Components/Blade/Blade.json'
    _type_info = TypeInfo(
        set([('Modelling', 'BladeModellingType'),]),
        set([]),
        set(['AerofoilLibrary','InterpolatedAerofoilLibrary','AileronAerofoilLibrary','CrossSections','OutputGroups',]),
        'ComponentType').merge(Component._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    @property
    def Modelling_as_FiniteElementBladeModelling(self) -> FiniteElementBladeModelling:
        """
        Retrieves the value of Modelling guaranteeing it is a FiniteElementBladeModelling; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        FiniteElementBladeModelling
            A model object, guaranteed to be a FiniteElementBladeModelling.

        Raises
        ------
        TypeError
            If the value is not a FiniteElementBladeModelling.
        """
        return self.Modelling_as(FiniteElementBladeModelling)


    @property
    def Modelling_as_ModalBladeModelling(self) -> ModalBladeModelling:
        """
        Retrieves the value of Modelling guaranteeing it is a ModalBladeModelling; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        ModalBladeModelling
            A model object, guaranteed to be a ModalBladeModelling.

        Raises
        ------
        TypeError
            If the value is not a ModalBladeModelling.
        """
        return self.Modelling_as(ModalBladeModelling)


    @property
    def Modelling_as_RigidBladeModelling(self) -> RigidBladeModelling:
        """
        Retrieves the value of Modelling guaranteeing it is a RigidBladeModelling; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        RigidBladeModelling
            A model object, guaranteed to be a RigidBladeModelling.

        Raises
        ------
        TypeError
            If the value is not a RigidBladeModelling.
        """
        return self.Modelling_as(RigidBladeModelling)


    @property
    def Modelling_as_inline(self) -> Union[FiniteElementBladeModelling, ModalBladeModelling, RigidBladeModelling]:
        """
        Retrieves the value of Modelling as a model object; if the value is specified with a '$insert', an error is raised.

        Returns
        -------
        Union[FiniteElementBladeModelling, ModalBladeModelling, RigidBladeModelling]
            A model object at the specified index, guaranteed to not be a '$insert'.

        Raises
        ------
        TypeError
            If the value is not one of the concrete types of BladeModelling; i.e. it is specified with a '$insert'.
        """
        if isinstance(self.Modelling, BladeModellingInsert) or self.Modelling.is_insert:
            raise TypeError(f"Expected Modelling value to be an in-line object, but it is currently in the '$insert' state.")
        return self.Modelling


    def Modelling_as(self, cls: Type[TBladeModellingOptions])-> TBladeModellingOptions:
        """
        Retrieves the value of Modelling, ensuring it is of the specified type.
        The specified type must be one of the valid concrete types of BladeModelling, or the '$insert' type
        (in the case the model is defined to be inserted from an external resource).

        Useful when using type-checking development tools, as it provides a concise way to access a value with the correct type.

        Parameters
        ----------
        cls: Type[Union[FiniteElementBladeModelling, BladeModellingInsert, ModalBladeModelling, RigidBladeModelling]]
            One of the valid concrete types of BladeModelling, or the '$insert' type
            (in the case the model is defined to be inserted from an external resource).

        Returns
        -------
        TBladeModellingOptions
            A model object of the specified type.

        Raises
        ------
        TypeError
            If the value is not of the specified type.

        Examples
        --------
        Get a reference to the value when it is one of the types of BladeModelling:
        >>> val_obj = model_obj.Modelling_as(models.FiniteElementBladeModelling)
        >>> val_obj = model_obj.Modelling_as(models.ModalBladeModelling)
        >>> val_obj = model_obj.Modelling_as(models.RigidBladeModelling)

        Get a reference to the value, when it was specified with a '$insert' and read in from a file:
        >>> insert_obj = model_obj.Modelling_as(models.BladeModellingInsert)
        """
        if not isinstance(self.Modelling, cls):
            raise TypeError(f"Expected Modelling of type '{cls.__name__}' but was type '{type(self.Modelling).__name__}'")
        return self.Modelling


    def _entity(self) -> bool:
        return True


Blade.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.bladed_model import BladedModel

from .schema_helper import SchemaHelper
from .models_impl import *


class FrequencyVsSpectralDensity(BladedModel):
    r"""
    A line or row in a look-up table, specifying a value of SpectralDensity for a specified Frequency.
    
    Not supported yet.
    
    Attributes
    ----------
    Frequency : float, Not supported yet
        The frequency of the waves.
    
    SpectralDensity : float, Not supported yet
        The power spectral density values associated with the defined frequency value.  The value of power spectral density at the lowest and highest frequencies entered should be zero.
    
    Notes
    -----
    This model is not supported yet by the Bladed calculation engine.
    """
    Frequency: float = Field(alias="Frequency", default=None) # Not supported yet
    SpectralDensity: float = Field(alias="SpectralDensity", default=None) # Not supported yet

    _relative_schema_path = 'TimeDomainSimulation/Environment/SeaState/Waves/FrequencyVsSpectralDensity/FrequencyVsSpectralDensity.json'
    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        None).merge(BladedModel._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    def _entity(self) -> bool:
        return True


FrequencyVsSpectralDensity.update_forward_refs()
